"""
Cognito resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_cognito_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Cognito resources: user pools, identity pools.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []

    # Cognito User Pools
    cognito_idp = session.client('cognito-idp', region_name=region)
    try:
        paginator = cognito_idp.get_paginator('list_user_pools')
        for page in paginator.paginate(MaxResults=60):
            for pool in page.get('UserPools', []):
                pool_id = pool['Id']
                pool_name = pool['Name']

                try:
                    # Get pool details
                    pool_response = cognito_idp.describe_user_pool(UserPoolId=pool_id)
                    pool_detail = pool_response.get('UserPool', {})

                    # Tags are in the detail response
                    tags = pool_detail.get('UserPoolTags', {})

                    resources.append({
                        'service': 'cognito',
                        'type': 'user-pool',
                        'id': pool_id,
                        'arn': pool_detail.get('Arn', f"arn:aws:cognito-idp:{region}:{account_id}:userpool/{pool_id}"),
                        'name': pool_name,
                        'region': region,
                        'details': {
                            'status': pool_detail.get('Status'),
                            'creation_date': str(pool_detail.get('CreationDate', '')),
                            'last_modified_date': str(pool_detail.get('LastModifiedDate', '')),
                            'mfa_configuration': pool_detail.get('MfaConfiguration'),
                            'estimated_number_of_users': pool_detail.get('EstimatedNumberOfUsers'),
                            'email_configuration_set': pool_detail.get('EmailConfiguration', {}).get('EmailSendingAccount'),
                            'sms_configuration': bool(pool_detail.get('SmsConfiguration')),
                            'user_pool_add_ons': pool_detail.get('UserPoolAddOns'),
                            'domain': pool_detail.get('Domain'),
                            'custom_domain': pool_detail.get('CustomDomain'),
                        },
                        'tags': tags
                    })

                    # User Pool Clients
                    try:
                        client_paginator = cognito_idp.get_paginator('list_user_pool_clients')
                        for client_page in client_paginator.paginate(UserPoolId=pool_id, MaxResults=60):
                            for client in client_page.get('UserPoolClients', []):
                                client_id = client['ClientId']
                                client_name = client['ClientName']

                                resources.append({
                                    'service': 'cognito',
                                    'type': 'user-pool-client',
                                    'id': client_id,
                                    'arn': f"arn:aws:cognito-idp:{region}:{account_id}:userpool/{pool_id}/client/{client_id}",
                                    'name': client_name,
                                    'region': region,
                                    'details': {
                                        'user_pool_id': pool_id,
                                        'user_pool_name': pool_name,
                                    },
                                    'tags': {}
                                })
                    except Exception:
                        pass

                except Exception:
                    pass
    except Exception:
        pass

    # Cognito Identity Pools
    cognito_identity = session.client('cognito-identity', region_name=region)
    try:
        paginator = cognito_identity.get_paginator('list_identity_pools')
        for page in paginator.paginate(MaxResults=60):
            for pool in page.get('IdentityPools', []):
                pool_id = pool['IdentityPoolId']
                pool_name = pool['IdentityPoolName']

                try:
                    # Get pool details
                    pool_response = cognito_identity.describe_identity_pool(IdentityPoolId=pool_id)

                    # Get tags
                    tags = {}
                    try:
                        tag_response = cognito_identity.list_tags_for_resource(
                            ResourceArn=f"arn:aws:cognito-identity:{region}:{account_id}:identitypool/{pool_id}"
                        )
                        tags = tag_response.get('Tags', {})
                    except Exception:
                        pass

                    resources.append({
                        'service': 'cognito',
                        'type': 'identity-pool',
                        'id': pool_id,
                        'arn': f"arn:aws:cognito-identity:{region}:{account_id}:identitypool/{pool_id}",
                        'name': pool_name,
                        'region': region,
                        'details': {
                            'allow_unauthenticated_identities': pool_response.get('AllowUnauthenticatedIdentities'),
                            'allow_classic_flow': pool_response.get('AllowClassicFlow'),
                            'developer_provider_name': pool_response.get('DeveloperProviderName'),
                            'cognito_identity_providers': [
                                p.get('ProviderName') for p in pool_response.get('CognitoIdentityProviders', [])
                            ],
                            'supported_login_providers': list(pool_response.get('SupportedLoginProviders', {}).keys()),
                            'saml_provider_arns': pool_response.get('SamlProviderARNs', []),
                            'open_id_connect_provider_arns': pool_response.get('OpenIdConnectProviderARNs', []),
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    return resources



"""
AWS Amazon Macie resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_macie2_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Amazon Macie resources: classification jobs, custom data identifiers,
    findings filters, allow lists, and members.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    macie = session.client('macie2', region_name=region)

    # Check if Macie is enabled
    try:
        status = macie.get_macie_session()
        if status.get('status') != 'ENABLED':
            return []
    except Exception:
        return []

    # Classification Jobs
    try:
        paginator = macie.get_paginator('list_classification_jobs')
        for page in paginator.paginate():
            for job in page.get('items', []):
                job_id = job['jobId']

                details = {
                    'job_type': job.get('jobType'),
                    'job_status': job.get('jobStatus'),
                    'name': job.get('name'),
                    'created_at': str(job.get('createdAt', '')),
                    'bucket_criteria_includes': job.get('bucketCriteria', {}).get('includes'),
                }

                # Get full job details
                try:
                    job_detail = macie.describe_classification_job(jobId=job_id)
                    details['description'] = job_detail.get('description')
                    details['sampling_percentage'] = job_detail.get('samplingPercentage')
                    details['initial_run'] = job_detail.get('initialRun')
                    details['last_run_time'] = str(job_detail.get('lastRunTime', ''))
                    details['managed_data_identifier_selector'] = job_detail.get('managedDataIdentifierSelector')

                    schedule = job_detail.get('scheduleFrequency', {})
                    if schedule:
                        details['schedule_frequency'] = schedule
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = macie.list_tags_for_resource(
                        resourceArn=f"arn:aws:macie2:{region}:{account_id}:classification-job/{job_id}"
                    )
                    tags = tag_response.get('tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'macie2',
                    'type': 'classification-job',
                    'id': job_id,
                    'arn': f"arn:aws:macie2:{region}:{account_id}:classification-job/{job_id}",
                    'name': job.get('name', job_id),
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Custom Data Identifiers
    try:
        paginator = macie.get_paginator('list_custom_data_identifiers')
        for page in paginator.paginate():
            for cdi in page.get('items', []):
                cdi_id = cdi['id']
                cdi_name = cdi.get('name', cdi_id)

                details = {
                    'description': cdi.get('description'),
                    'created_at': str(cdi.get('createdAt', '')),
                }

                # Get full details
                try:
                    cdi_detail = macie.get_custom_data_identifier(id=cdi_id)
                    details['regex'] = cdi_detail.get('regex')
                    details['keywords'] = cdi_detail.get('keywords', [])
                    details['ignore_words'] = cdi_detail.get('ignoreWords', [])
                    details['maximum_match_distance'] = cdi_detail.get('maximumMatchDistance')
                    details['severity_levels'] = cdi_detail.get('severityLevels', [])
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = macie.list_tags_for_resource(
                        resourceArn=cdi.get('arn', f"arn:aws:macie2:{region}:{account_id}:custom-data-identifier/{cdi_id}")
                    )
                    tags = tag_response.get('tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'macie2',
                    'type': 'custom-data-identifier',
                    'id': cdi_id,
                    'arn': cdi.get('arn', f"arn:aws:macie2:{region}:{account_id}:custom-data-identifier/{cdi_id}"),
                    'name': cdi_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Findings Filters
    try:
        paginator = macie.get_paginator('list_findings_filters')
        for page in paginator.paginate():
            for ff in page.get('findingsFilterListItems', []):
                ff_id = ff['id']
                ff_name = ff.get('name', ff_id)

                details = {
                    'action': ff.get('action'),
                }

                # Get full details
                try:
                    ff_detail = macie.get_findings_filter(id=ff_id)
                    details['description'] = ff_detail.get('description')
                    details['position'] = ff_detail.get('position')
                    details['finding_criteria'] = ff_detail.get('findingCriteria', {})
                except Exception:
                    pass

                # Get tags
                tags = ff.get('tags', {})

                resources.append({
                    'service': 'macie2',
                    'type': 'findings-filter',
                    'id': ff_id,
                    'arn': ff.get('arn', f"arn:aws:macie2:{region}:{account_id}:findings-filter/{ff_id}"),
                    'name': ff_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Allow Lists
    try:
        paginator = macie.get_paginator('list_allow_lists')
        for page in paginator.paginate():
            for al in page.get('allowLists', []):
                al_id = al['id']
                al_name = al.get('name', al_id)

                details = {
                    'description': al.get('description'),
                    'created_at': str(al.get('createdAt', '')),
                    'updated_at': str(al.get('updatedAt', '')),
                }

                # Get full details
                try:
                    al_detail = macie.get_allow_list(id=al_id)
                    criteria = al_detail.get('criteria', {})
                    if 's3WordsList' in criteria:
                        details['s3_bucket'] = criteria['s3WordsList'].get('bucketName')
                        details['s3_object_key'] = criteria['s3WordsList'].get('objectKey')
                    if 'regex' in criteria:
                        details['regex'] = criteria['regex']
                except Exception:
                    pass

                # Get tags
                tags = al.get('tags', {})

                resources.append({
                    'service': 'macie2',
                    'type': 'allow-list',
                    'id': al_id,
                    'arn': al.get('arn', f"arn:aws:macie2:{region}:{account_id}:allow-list/{al_id}"),
                    'name': al_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Members (multi-account)
    try:
        paginator = macie.get_paginator('list_members')
        for page in paginator.paginate():
            for member in page.get('members', []):
                member_id = member['accountId']

                details = {
                    'email': member.get('email'),
                    'relationship_status': member.get('relationshipStatus'),
                    'administrator_account_id': member.get('administratorAccountId'),
                    'master_account_id': member.get('masterAccountId'),
                    'invited_at': str(member.get('invitedAt', '')),
                    'updated_at': str(member.get('updatedAt', '')),
                }

                tags = member.get('tags', {})

                resources.append({
                    'service': 'macie2',
                    'type': 'member',
                    'id': member_id,
                    'arn': member.get('arn', f"arn:aws:macie2:{region}:{account_id}:member/{member_id}"),
                    'name': member_id,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
AWS Polly resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_polly_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Polly resources: lexicons.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    polly = session.client('polly', region_name=region)

    # Lexicons
    try:
        response = polly.list_lexicons()
        for lexicon in response.get('Lexicons', []):
            lexicon_name = lexicon['Name']
            attributes = lexicon.get('Attributes', {})

            details = {
                'alphabet': attributes.get('Alphabet'),
                'language_code': attributes.get('LanguageCode'),
                'last_modified': str(attributes.get('LastModified', '')),
                'lexicon_arn': attributes.get('LexiconArn'),
                'lexemes_count': attributes.get('LexemesCount'),
                'size': attributes.get('Size'),
            }

            resources.append({
                'service': 'polly',
                'type': 'lexicon',
                'id': lexicon_name,
                'arn': attributes.get('LexiconArn', f"arn:aws:polly:{region}:{account_id}:lexicon/{lexicon_name}"),
                'name': lexicon_name,
                'region': region,
                'details': details,
                'tags': {}
            })
    except Exception:
        pass

    return resources



"""
AWS Batch resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_batch_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Batch resources: compute environments, job queues, job definitions, scheduling policies.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    batch = session.client('batch', region_name=region)

    # Compute Environments
    try:
        paginator = batch.get_paginator('describe_compute_environments')
        for page in paginator.paginate():
            for ce in page.get('computeEnvironments', []):
                ce_name = ce['computeEnvironmentName']
                ce_arn = ce['computeEnvironmentArn']

                # Get tags
                tags = ce.get('tags', {})

                resources.append({
                    'service': 'batch',
                    'type': 'compute-environment',
                    'id': ce_name,
                    'arn': ce_arn,
                    'name': ce_name,
                    'region': region,
                    'details': {
                        'state': ce.get('state'),
                        'status': ce.get('status'),
                        'status_reason': ce.get('statusReason'),
                        'type': ce.get('type'),
                        'compute_resources': {
                            'type': ce.get('computeResources', {}).get('type'),
                            'allocation_strategy': ce.get('computeResources', {}).get('allocationStrategy'),
                            'min_vcpus': ce.get('computeResources', {}).get('minvCpus'),
                            'max_vcpus': ce.get('computeResources', {}).get('maxvCpus'),
                            'desired_vcpus': ce.get('computeResources', {}).get('desiredvCpus'),
                            'instance_types': ce.get('computeResources', {}).get('instanceTypes', []),
                        },
                        'service_role': ce.get('serviceRole'),
                        'update_policy': ce.get('updatePolicy'),
                        'eks_configuration': ce.get('eksConfiguration'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Job Queues
    try:
        paginator = batch.get_paginator('describe_job_queues')
        for page in paginator.paginate():
            for jq in page.get('jobQueues', []):
                jq_name = jq['jobQueueName']
                jq_arn = jq['jobQueueArn']

                # Get tags
                tags = jq.get('tags', {})

                resources.append({
                    'service': 'batch',
                    'type': 'job-queue',
                    'id': jq_name,
                    'arn': jq_arn,
                    'name': jq_name,
                    'region': region,
                    'details': {
                        'state': jq.get('state'),
                        'status': jq.get('status'),
                        'status_reason': jq.get('statusReason'),
                        'priority': jq.get('priority'),
                        'scheduling_policy_arn': jq.get('schedulingPolicyArn'),
                        'compute_environment_order': [
                            {
                                'order': ceo.get('order'),
                                'compute_environment': ceo.get('computeEnvironment'),
                            }
                            for ceo in jq.get('computeEnvironmentOrder', [])
                        ],
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Job Definitions (active only, latest revision)
    try:
        paginator = batch.get_paginator('describe_job_definitions')
        for page in paginator.paginate(status='ACTIVE'):
            for jd in page.get('jobDefinitions', []):
                jd_name = jd['jobDefinitionName']
                jd_arn = jd['jobDefinitionArn']

                # Get tags
                tags = jd.get('tags', {})

                resources.append({
                    'service': 'batch',
                    'type': 'job-definition',
                    'id': f"{jd_name}:{jd.get('revision')}",
                    'arn': jd_arn,
                    'name': jd_name,
                    'region': region,
                    'details': {
                        'revision': jd.get('revision'),
                        'status': jd.get('status'),
                        'type': jd.get('type'),
                        'scheduling_priority': jd.get('schedulingPriority'),
                        'platform_capabilities': jd.get('platformCapabilities', []),
                        'propagate_tags': jd.get('propagateTags'),
                        'timeout': jd.get('timeout'),
                        'retry_strategy': jd.get('retryStrategy'),
                        'container_orchestration_type': jd.get('containerOrchestrationType'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Scheduling Policies
    try:
        paginator = batch.get_paginator('list_scheduling_policies')
        for page in paginator.paginate():
            sp_arns = [sp['arn'] for sp in page.get('schedulingPolicies', [])]

            if sp_arns:
                # Describe scheduling policies
                desc_response = batch.describe_scheduling_policies(arns=sp_arns)
                for sp in desc_response.get('schedulingPolicies', []):
                    sp_name = sp['name']
                    sp_arn = sp['arn']

                    tags = sp.get('tags', {})

                    resources.append({
                        'service': 'batch',
                        'type': 'scheduling-policy',
                        'id': sp_name,
                        'arn': sp_arn,
                        'name': sp_name,
                        'region': region,
                        'details': {
                            'fairshare_policy': sp.get('fairsharePolicy'),
                        },
                        'tags': tags
                    })
    except Exception:
        pass

    return resources



"""
Amazon EventBridge Pipes resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_pipes_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect EventBridge Pipes resources.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    pipes = session.client('pipes', region_name=region)

    # Pipes
    try:
        paginator = pipes.get_paginator('list_pipes')
        for page in paginator.paginate():
            for pipe in page.get('Pipes', []):
                pipe_arn = pipe.get('Arn', '')
                pipe_name = pipe.get('Name', pipe_arn.split('/')[-1])

                details = {
                    'current_state': pipe.get('CurrentState'),
                    'desired_state': pipe.get('DesiredState'),
                    'source': pipe.get('Source'),
                    'target': pipe.get('Target'),
                    'enrichment': pipe.get('Enrichment'),
                    'creation_time': str(pipe.get('CreationTime', '')) if pipe.get('CreationTime') else None,
                }

                resources.append({
                    'service': 'pipes',
                    'type': 'pipe',
                    'id': pipe_name,
                    'arn': pipe_arn,
                    'name': pipe_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
Athena resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_athena_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Athena resources: workgroups, data catalogs, named queries.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    athena = session.client('athena', region_name=region)

    # Workgroups
    try:
        paginator = athena.get_paginator('list_work_groups')
        for page in paginator.paginate():
            for wg in page.get('WorkGroups', []):
                wg_name = wg['Name']

                try:
                    # Get workgroup details
                    wg_response = athena.get_work_group(WorkGroup=wg_name)
                    wg_detail = wg_response.get('WorkGroup', {})

                    wg_arn = f"arn:aws:athena:{region}:{account_id}:workgroup/{wg_name}"

                    # Get tags
                    tags = {}
                    try:
                        tag_response = athena.list_tags_for_resource(ResourceARN=wg_arn)
                        for tag in tag_response.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                    except Exception:
                        pass

                    config = wg_detail.get('Configuration', {})

                    resources.append({
                        'service': 'athena',
                        'type': 'workgroup',
                        'id': wg_name,
                        'arn': wg_arn,
                        'name': wg_name,
                        'region': region,
                        'details': {
                            'state': wg_detail.get('State'),
                            'description': wg_detail.get('Description'),
                            'creation_time': str(wg_detail.get('CreationTime', '')),
                            'engine_version': config.get('EngineVersion', {}).get('SelectedEngineVersion'),
                            'result_output_location': config.get('ResultConfiguration', {}).get('OutputLocation'),
                            'enforce_workgroup_configuration': config.get('EnforceWorkGroupConfiguration'),
                            'publish_cloudwatch_metrics_enabled': config.get('PublishCloudWatchMetricsEnabled'),
                            'bytes_scanned_cutoff_per_query': config.get('BytesScannedCutoffPerQuery'),
                            'requester_pays_enabled': config.get('RequesterPaysEnabled'),
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    # Data Catalogs (non-default)
    try:
        paginator = athena.get_paginator('list_data_catalogs')
        for page in paginator.paginate():
            for catalog in page.get('DataCatalogsSummary', []):
                catalog_name = catalog['CatalogName']

                # Skip the default AWS Glue catalog
                if catalog_name == 'AwsDataCatalog':
                    continue

                try:
                    # Get catalog details
                    catalog_response = athena.get_data_catalog(Name=catalog_name)
                    catalog_detail = catalog_response.get('DataCatalog', {})

                    catalog_arn = f"arn:aws:athena:{region}:{account_id}:datacatalog/{catalog_name}"

                    # Get tags
                    tags = {}
                    try:
                        tag_response = athena.list_tags_for_resource(ResourceARN=catalog_arn)
                        for tag in tag_response.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                    except Exception:
                        pass

                    resources.append({
                        'service': 'athena',
                        'type': 'data-catalog',
                        'id': catalog_name,
                        'arn': catalog_arn,
                        'name': catalog_name,
                        'region': region,
                        'details': {
                            'type': catalog_detail.get('Type'),
                            'description': catalog_detail.get('Description'),
                        },
                        'tags': tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    # Named Queries
    try:
        paginator = athena.get_paginator('list_named_queries')
        for page in paginator.paginate():
            query_ids = page.get('NamedQueryIds', [])

            # Batch get named queries (max 50 at a time)
            for i in range(0, len(query_ids), 50):
                batch = query_ids[i:i+50]
                try:
                    response = athena.batch_get_named_query(NamedQueryIds=batch)
                    for query in response.get('NamedQueries', []):
                        query_id = query['NamedQueryId']
                        query_name = query['Name']

                        resources.append({
                            'service': 'athena',
                            'type': 'named-query',
                            'id': query_id,
                            'arn': f"arn:aws:athena:{region}:{account_id}:namedquery/{query_id}",
                            'name': query_name,
                            'region': region,
                            'details': {
                                'database': query.get('Database'),
                                'description': query.get('Description'),
                                'workgroup': query.get('WorkGroup'),
                            },
                            'tags': {}
                        })
                except Exception:
                    pass
    except Exception:
        pass

    # Note: Prepared statements skipped for performance (requires N×M API calls per workgroup)

    return resources



"""
AWS Translate resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# Translate supported regions (from https://docs.aws.amazon.com/general/latest/gr/translate-service.html)
TRANSLATE_REGIONS = {
    'us-east-1', 'us-east-2', 'us-west-1', 'us-west-2',
    'ap-east-1', 'ap-south-1', 'ap-northeast-1', 'ap-northeast-2', 'ap-southeast-1', 'ap-southeast-2',
    'ca-central-1',
    'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-west-3', 'eu-north-1',
}


def collect_translate_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Translate resources: terminologies and parallel data.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in TRANSLATE_REGIONS:
        return []

    resources = []
    translate = session.client('translate', region_name=region)

    # Terminologies
    try:
        paginator = translate.get_paginator('list_terminologies')
        for page in paginator.paginate():
            for term in page.get('TerminologyPropertiesList', []):
                term_name = term['Name']
                term_arn = term.get('Arn', '')

                details = {
                    'source_language_code': term.get('SourceLanguageCode'),
                    'target_language_codes': term.get('TargetLanguageCodes', []),
                    'description': term.get('Description'),
                    'term_count': term.get('TermCount'),
                    'size_bytes': term.get('SizeBytes'),
                    'directionality': term.get('Directionality'),
                    'format': term.get('Format'),
                    'created_at': str(term.get('CreatedAt', '')),
                    'last_updated_at': str(term.get('LastUpdatedAt', '')),
                }

                resources.append({
                    'service': 'translate',
                    'type': 'terminology',
                    'id': term_name,
                    'arn': term_arn,
                    'name': term_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Parallel Data
    try:
        paginator = translate.get_paginator('list_parallel_data')
        for page in paginator.paginate():
            for pd in page.get('ParallelDataPropertiesList', []):
                pd_name = pd['Name']
                pd_arn = pd.get('Arn', '')

                details = {
                    'status': pd.get('Status'),
                    'source_language_code': pd.get('SourceLanguageCode'),
                    'target_language_codes': pd.get('TargetLanguageCodes', []),
                    'description': pd.get('Description'),
                    'parallel_data_config': pd.get('ParallelDataConfig', {}).get('Format'),
                    'imported_data_size': pd.get('ImportedDataSize'),
                    'imported_record_count': pd.get('ImportedRecordCount'),
                    'failed_record_count': pd.get('FailedRecordCount'),
                    'skipped_record_count': pd.get('SkippedRecordCount'),
                    'created_at': str(pd.get('CreatedAt', '')),
                    'last_updated_at': str(pd.get('LastUpdatedAt', '')),
                    'message': pd.get('Message'),
                }

                resources.append({
                    'service': 'translate',
                    'type': 'parallel-data',
                    'id': pd_name,
                    'arn': pd_arn,
                    'name': pd_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Health resource collector.

Note: AWS Health API requires Business, Enterprise On-Ramp, or Enterprise Support plan.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_health_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Health events affecting the account.

    Note: Requires Business or Enterprise Support plan. Returns empty if not available.

    Args:
        session: boto3.Session to use
        region: AWS region (ignored - Health is a global service using us-east-1)
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    # Health API must be called from us-east-1
    health = session.client('health', region_name='us-east-1')

    # Events (open events only, not historical)
    try:
        paginator = health.get_paginator('describe_events')
        for page in paginator.paginate(
            filter={
                'eventStatusCodes': ['open', 'upcoming']
            }
        ):
            for event in page.get('events', []):
                event_arn = event.get('arn', '')
                event_type = event.get('eventTypeCode', '')

                details = {
                    'service': event.get('service'),
                    'event_type_code': event_type,
                    'event_type_category': event.get('eventTypeCategory'),
                    'event_scope_code': event.get('eventScopeCode'),
                    'availability_zone': event.get('availabilityZone'),
                    'start_time': str(event.get('startTime', '')),
                    'end_time': str(event.get('endTime', '')),
                    'last_updated_time': str(event.get('lastUpdatedTime', '')),
                    'status_code': event.get('statusCode'),
                }

                # Get event details
                try:
                    detail_response = health.describe_event_details(eventArns=[event_arn])
                    for detail in detail_response.get('successfulSet', []):
                        event_detail = detail.get('event', {})
                        event_description = detail.get('eventDescription', {})
                        details['description'] = event_description.get('latestDescription')
                except Exception:
                    pass

                # Get affected entities count
                try:
                    entities_response = health.describe_affected_entities(
                        filter={'eventArns': [event_arn]}
                    )
                    entities = entities_response.get('entities', [])
                    details['affected_entities_count'] = len(entities)
                    details['affected_entity_values'] = [e.get('entityValue') for e in entities[:10]]
                except Exception:
                    pass

                resources.append({
                    'service': 'health',
                    'type': 'event',
                    'id': event_arn.split('/')[-1] if event_arn else event_type,
                    'arn': event_arn,
                    'name': event_type,
                    'region': event.get('region', 'global'),
                    'details': details,
                    'tags': {}
                })
    except health.exceptions.SubscriptionRequiredException:
        # Account doesn't have Business/Enterprise Support
        return []
    except Exception:
        pass

    return resources



"""
AWS Service Quotas resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_servicequotas_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Service Quotas resources: quota increase requests.

    Note: Only collects quota increase requests history, not all quotas
    (which would be thousands of resources).

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    sq = session.client('service-quotas', region_name=region)

    # Quota Increase Requests History
    try:
        paginator = sq.get_paginator('list_requested_service_quota_change_history')
        for page in paginator.paginate():
            for request in page.get('RequestedQuotas', []):
                request_id = request.get('Id', '')
                quota_code = request.get('QuotaCode', '')
                service_code = request.get('ServiceCode', '')

                details = {
                    'service_code': service_code,
                    'service_name': request.get('ServiceName'),
                    'quota_code': quota_code,
                    'quota_name': request.get('QuotaName'),
                    'status': request.get('Status'),
                    'desired_value': request.get('DesiredValue'),
                    'case_id': request.get('CaseId'),
                    'created': str(request.get('Created', '')),
                    'last_updated': str(request.get('LastUpdated', '')),
                    'requester': request.get('Requester'),
                    'quota_arn': request.get('QuotaArn'),
                    'global_quota': request.get('GlobalQuota'),
                    'unit': request.get('Unit'),
                }

                resources.append({
                    'service': 'service-quotas',
                    'type': 'quota-request',
                    'id': request_id,
                    'arn': request.get('QuotaArn', f"arn:aws:servicequotas:{region}:{account_id}:request/{request_id}"),
                    'name': f"{service_code}/{quota_code}",
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
API Gateway v2 (HTTP/WebSocket APIs) resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_apigatewayv2_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect API Gateway v2 resources: HTTP APIs, WebSocket APIs, stages.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    apigwv2 = session.client('apigatewayv2', region_name=region)

    # HTTP and WebSocket APIs
    try:
        paginator = apigwv2.get_paginator('get_apis')
        for page in paginator.paginate():
            for api in page.get('Items', []):
                api_id = api['ApiId']
                api_name = api.get('Name', api_id)

                # Get tags
                tags = api.get('Tags', {})

                api_type = api.get('ProtocolType', 'HTTP')

                resources.append({
                    'service': 'apigatewayv2',
                    'type': f"{api_type.lower()}-api",
                    'id': api_id,
                    'arn': f"arn:aws:apigateway:{region}::/apis/{api_id}",
                    'name': api_name,
                    'region': region,
                    'details': {
                        'protocol_type': api_type,
                        'description': api.get('Description'),
                        'api_endpoint': api.get('ApiEndpoint'),
                        'created_date': str(api.get('CreatedDate', '')),
                        'version': api.get('Version'),
                        'route_selection_expression': api.get('RouteSelectionExpression'),
                        'api_gateway_managed': api.get('ApiGatewayManaged'),
                        'disable_execute_api_endpoint': api.get('DisableExecuteApiEndpoint'),
                    },
                    'tags': tags
                })

                # Stages for this API
                try:
                    stages_paginator = apigwv2.get_paginator('get_stages')
                    for stages_page in stages_paginator.paginate(ApiId=api_id):
                        for stage in stages_page.get('Items', []):
                            stage_name = stage['StageName']

                            stage_tags = stage.get('Tags', {})

                            resources.append({
                                'service': 'apigatewayv2',
                                'type': 'stage',
                                'id': f"{api_id}/{stage_name}",
                                'arn': f"arn:aws:apigateway:{region}::/apis/{api_id}/stages/{stage_name}",
                                'name': f"{api_name}/{stage_name}",
                                'region': region,
                                'details': {
                                    'api_id': api_id,
                                    'api_name': api_name,
                                    'deployment_id': stage.get('DeploymentId'),
                                    'description': stage.get('Description'),
                                    'auto_deploy': stage.get('AutoDeploy'),
                                    'created_date': str(stage.get('CreatedDate', '')),
                                    'last_updated_date': str(stage.get('LastUpdatedDate', '')),
                                    'default_route_settings': stage.get('DefaultRouteSettings'),
                                },
                                'tags': stage_tags
                            })
                except Exception:
                    pass
    except Exception:
        pass

    # VPC Links (v2)
    try:
        paginator = apigwv2.get_paginator('get_vpc_links')
        for page in paginator.paginate():
            for vpc_link in page.get('Items', []):
                vl_id = vpc_link['VpcLinkId']
                vl_name = vpc_link.get('Name', vl_id)

                tags = vpc_link.get('Tags', {})

                resources.append({
                    'service': 'apigatewayv2',
                    'type': 'vpc-link',
                    'id': vl_id,
                    'arn': f"arn:aws:apigateway:{region}::/vpclinks/{vl_id}",
                    'name': vl_name,
                    'region': region,
                    'details': {
                        'status': vpc_link.get('VpcLinkStatus'),
                        'status_message': vpc_link.get('VpcLinkStatusMessage'),
                        'vpc_link_version': vpc_link.get('VpcLinkVersion'),
                        'subnet_ids': vpc_link.get('SubnetIds', []),
                        'security_group_ids': vpc_link.get('SecurityGroupIds', []),
                        'created_date': str(vpc_link.get('CreatedDate', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Domain Names
    try:
        paginator = apigwv2.get_paginator('get_domain_names')
        for page in paginator.paginate():
            for domain in page.get('Items', []):
                domain_name = domain['DomainName']

                tags = domain.get('Tags', {})

                resources.append({
                    'service': 'apigatewayv2',
                    'type': 'domain-name',
                    'id': domain_name,
                    'arn': f"arn:aws:apigateway:{region}::/domainnames/{domain_name}",
                    'name': domain_name,
                    'region': region,
                    'details': {
                        'domain_name_configurations': domain.get('DomainNameConfigurations', []),
                        'mutual_tls_authentication': domain.get('MutualTlsAuthentication'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
AWS Service Catalog resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_servicecatalog_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Service Catalog resources: portfolios and products.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    sc = session.client('servicecatalog', region_name=region)

    # Portfolios
    try:
        paginator = sc.get_paginator('list_portfolios')
        for page in paginator.paginate():
            for portfolio in page.get('PortfolioDetails', []):
                portfolio_id = portfolio['Id']
                portfolio_arn = portfolio.get('ARN', f"arn:aws:catalog:{region}:{account_id}:portfolio/{portfolio_id}")
                portfolio_name = portfolio.get('DisplayName', portfolio_id)

                details = {
                    'description': portfolio.get('Description'),
                    'provider_name': portfolio.get('ProviderName'),
                }

                resources.append({
                    'service': 'servicecatalog',
                    'type': 'portfolio',
                    'id': portfolio_id,
                    'arn': portfolio_arn,
                    'name': portfolio_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Products (as admin)
    try:
        paginator = sc.get_paginator('search_products_as_admin')
        for page in paginator.paginate():
            for product_detail in page.get('ProductViewDetails', []):
                product_arn = product_detail.get('ProductARN', '')
                status = product_detail.get('Status')

                summary = product_detail.get('ProductViewSummary', {})
                product_id = summary.get('ProductId', '')
                product_name = summary.get('Name', product_id)

                details = {
                    'status': status,
                    'type': summary.get('Type'),
                    'owner': summary.get('Owner'),
                    'short_description': summary.get('ShortDescription'),
                    'distributor': summary.get('Distributor'),
                    'has_default_path': summary.get('HasDefaultPath'),
                    'support_email': summary.get('SupportEmail'),
                }

                resources.append({
                    'service': 'servicecatalog',
                    'type': 'product',
                    'id': product_id,
                    'arn': product_arn,
                    'name': product_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
Glue resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_glue_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Glue resources: databases, tables, jobs, crawlers, connections.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    glue = session.client('glue', region_name=region)

    # Glue Databases
    databases = []
    try:
        paginator = glue.get_paginator('get_databases')
        for page in paginator.paginate():
            for db in page.get('DatabaseList', []):
                databases.append(db)
                db_name = db['Name']

                resources.append({
                    'service': 'glue',
                    'type': 'database',
                    'id': db_name,
                    'arn': f"arn:aws:glue:{region}:{account_id}:database/{db_name}",
                    'name': db_name,
                    'region': region,
                    'details': {
                        'description': db.get('Description'),
                        'location_uri': db.get('LocationUri'),
                        'create_time': str(db.get('CreateTime', '')),
                        'catalog_id': db.get('CatalogId'),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    # Glue Tables (for each database)
    for db in databases:
        db_name = db['Name']
        try:
            paginator = glue.get_paginator('get_tables')
            for page in paginator.paginate(DatabaseName=db_name):
                for table in page.get('TableList', []):
                    table_name = table['Name']

                    resources.append({
                        'service': 'glue',
                        'type': 'table',
                        'id': f"{db_name}/{table_name}",
                        'arn': f"arn:aws:glue:{region}:{account_id}:table/{db_name}/{table_name}",
                        'name': table_name,
                        'region': region,
                        'details': {
                            'database_name': db_name,
                            'description': table.get('Description'),
                            'table_type': table.get('TableType'),
                            'owner': table.get('Owner'),
                            'create_time': str(table.get('CreateTime', '')),
                            'update_time': str(table.get('UpdateTime', '')),
                            'last_access_time': str(table.get('LastAccessTime', '')),
                            'retention': table.get('Retention'),
                            'storage_descriptor_location': table.get('StorageDescriptor', {}).get('Location'),
                            'storage_descriptor_input_format': table.get('StorageDescriptor', {}).get('InputFormat'),
                            'columns_count': len(table.get('StorageDescriptor', {}).get('Columns', [])),
                            'partition_keys_count': len(table.get('PartitionKeys', [])),
                        },
                        'tags': {}
                    })
        except Exception:
            pass

    # Glue Jobs
    try:
        paginator = glue.get_paginator('get_jobs')
        for page in paginator.paginate():
            for job in page.get('Jobs', []):
                job_name = job['Name']

                # Get tags
                tags = {}
                try:
                    tag_response = glue.get_tags(
                        ResourceArn=f"arn:aws:glue:{region}:{account_id}:job/{job_name}"
                    )
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'glue',
                    'type': 'job',
                    'id': job_name,
                    'arn': f"arn:aws:glue:{region}:{account_id}:job/{job_name}",
                    'name': job_name,
                    'region': region,
                    'details': {
                        'description': job.get('Description'),
                        'role': job.get('Role'),
                        'created_on': str(job.get('CreatedOn', '')),
                        'last_modified_on': str(job.get('LastModifiedOn', '')),
                        'glue_version': job.get('GlueVersion'),
                        'worker_type': job.get('WorkerType'),
                        'number_of_workers': job.get('NumberOfWorkers'),
                        'max_capacity': job.get('MaxCapacity'),
                        'max_retries': job.get('MaxRetries'),
                        'timeout': job.get('Timeout'),
                        'execution_class': job.get('ExecutionClass'),
                        'command_name': job.get('Command', {}).get('Name'),
                        'command_python_version': job.get('Command', {}).get('PythonVersion'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Glue Crawlers
    try:
        paginator = glue.get_paginator('get_crawlers')
        for page in paginator.paginate():
            for crawler in page.get('Crawlers', []):
                crawler_name = crawler['Name']

                # Get tags
                tags = {}
                try:
                    tag_response = glue.get_tags(
                        ResourceArn=f"arn:aws:glue:{region}:{account_id}:crawler/{crawler_name}"
                    )
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'glue',
                    'type': 'crawler',
                    'id': crawler_name,
                    'arn': f"arn:aws:glue:{region}:{account_id}:crawler/{crawler_name}",
                    'name': crawler_name,
                    'region': region,
                    'details': {
                        'role': crawler.get('Role'),
                        'database_name': crawler.get('DatabaseName'),
                        'description': crawler.get('Description'),
                        'state': crawler.get('State'),
                        'table_prefix': crawler.get('TablePrefix'),
                        'schedule': crawler.get('Schedule', {}).get('ScheduleExpression'),
                        'schedule_state': crawler.get('Schedule', {}).get('State'),
                        'crawl_elapsed_time': crawler.get('CrawlElapsedTime'),
                        'creation_time': str(crawler.get('CreationTime', '')),
                        'last_updated': str(crawler.get('LastUpdated', '')),
                        'last_crawl_status': crawler.get('LastCrawl', {}).get('Status'),
                        'version': crawler.get('Version'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Glue Connections
    try:
        paginator = glue.get_paginator('get_connections')
        for page in paginator.paginate():
            for conn in page.get('ConnectionList', []):
                conn_name = conn['Name']

                resources.append({
                    'service': 'glue',
                    'type': 'connection',
                    'id': conn_name,
                    'arn': f"arn:aws:glue:{region}:{account_id}:connection/{conn_name}",
                    'name': conn_name,
                    'region': region,
                    'details': {
                        'connection_type': conn.get('ConnectionType'),
                        'description': conn.get('Description'),
                        'creation_time': str(conn.get('CreationTime', '')),
                        'last_updated_time': str(conn.get('LastUpdatedTime', '')),
                        'physical_connection_requirements': bool(conn.get('PhysicalConnectionRequirements')),
                    },
                    'tags': {}
                })
    except Exception:
        pass

    # Glue Registries
    try:
        paginator = glue.get_paginator('list_registries')
        for page in paginator.paginate():
            for registry in page.get('Registries', []):
                registry_name = registry['RegistryName']
                registry_arn = registry['RegistryArn']

                # Get tags
                tags = {}
                try:
                    tag_response = glue.get_tags(ResourceArn=registry_arn)
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'glue',
                    'type': 'registry',
                    'id': registry_name,
                    'arn': registry_arn,
                    'name': registry_name,
                    'region': region,
                    'details': {
                        'status': registry.get('Status'),
                        'description': registry.get('Description'),
                        'created_time': str(registry.get('CreatedTime', '')),
                        'updated_time': str(registry.get('UpdatedTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
DocumentDB resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_docdb_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect DocumentDB resources: clusters, instances, cluster snapshots.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    docdb = session.client('docdb', region_name=region)

    # DocumentDB Clusters
    try:
        paginator = docdb.get_paginator('describe_db_clusters')
        for page in paginator.paginate(Filters=[{'Name': 'engine', 'Values': ['docdb']}]):
            for cluster in page.get('DBClusters', []):
                cluster_id = cluster['DBClusterIdentifier']
                cluster_arn = cluster['DBClusterArn']

                # Get tags
                tags = {}
                try:
                    tag_response = docdb.list_tags_for_resource(ResourceName=cluster_arn)
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'docdb',
                    'type': 'cluster',
                    'id': cluster_id,
                    'arn': cluster_arn,
                    'name': cluster_id,
                    'region': region,
                    'details': {
                        'status': cluster.get('Status'),
                        'engine': cluster.get('Engine'),
                        'engine_version': cluster.get('EngineVersion'),
                        'endpoint': cluster.get('Endpoint'),
                        'reader_endpoint': cluster.get('ReaderEndpoint'),
                        'port': cluster.get('Port'),
                        'master_username': cluster.get('MasterUsername'),
                        'multi_az': cluster.get('MultiAZ'),
                        'storage_encrypted': cluster.get('StorageEncrypted'),
                        'kms_key_id': cluster.get('KmsKeyId'),
                        'db_cluster_members': len(cluster.get('DBClusterMembers', [])),
                        'backup_retention_period': cluster.get('BackupRetentionPeriod'),
                        'deletion_protection': cluster.get('DeletionProtection'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # DocumentDB Instances
    try:
        paginator = docdb.get_paginator('describe_db_instances')
        for page in paginator.paginate(Filters=[{'Name': 'engine', 'Values': ['docdb']}]):
            for instance in page.get('DBInstances', []):
                instance_id = instance['DBInstanceIdentifier']
                instance_arn = instance['DBInstanceArn']

                # Get tags
                tags = {}
                try:
                    tag_response = docdb.list_tags_for_resource(ResourceName=instance_arn)
                    for tag in tag_response.get('TagList', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'docdb',
                    'type': 'instance',
                    'id': instance_id,
                    'arn': instance_arn,
                    'name': instance_id,
                    'region': region,
                    'details': {
                        'cluster_identifier': instance.get('DBClusterIdentifier'),
                        'instance_class': instance.get('DBInstanceClass'),
                        'status': instance.get('DBInstanceStatus'),
                        'engine': instance.get('Engine'),
                        'engine_version': instance.get('EngineVersion'),
                        'endpoint': instance.get('Endpoint', {}).get('Address'),
                        'port': instance.get('Endpoint', {}).get('Port'),
                        'availability_zone': instance.get('AvailabilityZone'),
                        'publicly_accessible': instance.get('PubliclyAccessible'),
                        'storage_encrypted': instance.get('StorageEncrypted'),
                        'ca_certificate_identifier': instance.get('CACertificateIdentifier'),
                        'promotion_tier': instance.get('PromotionTier'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
AWS AppSync resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_appsync_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS AppSync resources: GraphQL APIs, data sources, functions,
    resolvers, API keys, and domain names.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    appsync = session.client('appsync', region_name=region)

    # GraphQL APIs
    api_ids = []
    try:
        paginator = appsync.get_paginator('list_graphql_apis')
        for page in paginator.paginate():
            for api in page.get('graphqlApis', []):
                api_id = api['apiId']
                api_ids.append(api_id)
                api_arn = api.get('arn', f"arn:aws:appsync:{region}:{account_id}:apis/{api_id}")
                api_name = api.get('name', api_id)

                details = {
                    'api_type': api.get('apiType'),
                    'authentication_type': api.get('authenticationType'),
                    'uris': api.get('uris', {}),
                    'xray_enabled': api.get('xrayEnabled'),
                    'waf_web_acl_arn': api.get('wafWebAclArn'),
                    'visibility': api.get('visibility'),
                    'owner': api.get('owner'),
                    'owner_contact': api.get('ownerContact'),
                    'introspection_config': api.get('introspectionConfig'),
                    'query_depth_limit': api.get('queryDepthLimit'),
                    'resolver_count_limit': api.get('resolverCountLimit'),
                    'enhanced_metrics_config': api.get('enhancedMetricsConfig'),
                }

                # Get additional auth providers
                additional_auth = api.get('additionalAuthenticationProviders', [])
                if additional_auth:
                    details['additional_auth_types'] = [a.get('authenticationType') for a in additional_auth]

                # Get log config
                log_config = api.get('logConfig', {})
                if log_config:
                    details['cloudwatch_logs_role_arn'] = log_config.get('cloudWatchLogsRoleArn')
                    details['field_log_level'] = log_config.get('fieldLogLevel')
                    details['exclude_verbose_content'] = log_config.get('excludeVerboseContent')

                # Get user pool config
                user_pool = api.get('userPoolConfig', {})
                if user_pool:
                    details['user_pool_id'] = user_pool.get('userPoolId')
                    details['user_pool_region'] = user_pool.get('awsRegion')

                # Get lambda authorizer config
                lambda_auth = api.get('lambdaAuthorizerConfig', {})
                if lambda_auth:
                    details['lambda_authorizer_uri'] = lambda_auth.get('authorizerUri')

                # Get cache info
                try:
                    cache_response = appsync.get_api_cache(apiId=api_id)
                    cache = cache_response.get('apiCache', {})
                    if cache:
                        details['cache_type'] = cache.get('type')
                        details['cache_ttl'] = cache.get('ttl')
                        details['cache_status'] = cache.get('status')
                        details['cache_at_rest_encryption'] = cache.get('atRestEncryptionEnabled')
                        details['cache_transit_encryption'] = cache.get('transitEncryptionEnabled')
                except Exception:
                    pass

                # Get tags
                tags = api.get('tags', {})

                resources.append({
                    'service': 'appsync',
                    'type': 'graphql-api',
                    'id': api_id,
                    'arn': api_arn,
                    'name': api_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Data Sources (per API)
    for api_id in api_ids:
        try:
            paginator = appsync.get_paginator('list_data_sources')
            for page in paginator.paginate(apiId=api_id):
                for ds in page.get('dataSources', []):
                    ds_name = ds['name']
                    ds_arn = ds.get('dataSourceArn', f"arn:aws:appsync:{region}:{account_id}:apis/{api_id}/datasources/{ds_name}")

                    details = {
                        'api_id': api_id,
                        'type': ds.get('type'),
                        'description': ds.get('description'),
                        'service_role_arn': ds.get('serviceRoleArn'),
                    }

                    # Type-specific config
                    ds_type = ds.get('type', '')
                    if ds_type == 'AWS_LAMBDA':
                        lambda_config = ds.get('lambdaConfig', {})
                        details['lambda_function_arn'] = lambda_config.get('lambdaFunctionArn')
                    elif ds_type == 'AMAZON_DYNAMODB':
                        dynamodb_config = ds.get('dynamodbConfig', {})
                        details['dynamodb_table_name'] = dynamodb_config.get('tableName')
                        details['dynamodb_region'] = dynamodb_config.get('awsRegion')
                        details['dynamodb_use_caller_credentials'] = dynamodb_config.get('useCallerCredentials')
                    elif ds_type == 'AMAZON_ELASTICSEARCH' or ds_type == 'AMAZON_OPENSEARCH_SERVICE':
                        es_config = ds.get('elasticsearchConfig') or ds.get('openSearchServiceConfig', {})
                        details['elasticsearch_endpoint'] = es_config.get('endpoint')
                        details['elasticsearch_region'] = es_config.get('awsRegion')
                    elif ds_type == 'HTTP':
                        http_config = ds.get('httpConfig', {})
                        details['http_endpoint'] = http_config.get('endpoint')
                    elif ds_type == 'RELATIONAL_DATABASE':
                        rds_config = ds.get('relationalDatabaseConfig', {})
                        details['rds_source_type'] = rds_config.get('relationalDatabaseSourceType')
                        rds_http = rds_config.get('rdsHttpEndpointConfig', {})
                        details['rds_cluster_arn'] = rds_http.get('dbClusterIdentifier')
                        details['rds_database_name'] = rds_http.get('databaseName')
                    elif ds_type == 'AMAZON_EVENTBRIDGE':
                        eb_config = ds.get('eventBridgeConfig', {})
                        details['eventbridge_bus_arn'] = eb_config.get('eventBusArn')

                    resources.append({
                        'service': 'appsync',
                        'type': 'data-source',
                        'id': f"{api_id}/{ds_name}",
                        'arn': ds_arn,
                        'name': ds_name,
                        'region': region,
                        'details': details,
                        'tags': {}
                    })
        except Exception:
            pass

    # Functions (per API)
    for api_id in api_ids:
        try:
            paginator = appsync.get_paginator('list_functions')
            for page in paginator.paginate(apiId=api_id):
                for func in page.get('functions', []):
                    func_id = func['functionId']
                    func_name = func.get('name', func_id)
                    func_arn = func.get('functionArn', f"arn:aws:appsync:{region}:{account_id}:apis/{api_id}/functions/{func_id}")

                    details = {
                        'api_id': api_id,
                        'description': func.get('description'),
                        'data_source_name': func.get('dataSourceName'),
                        'function_version': func.get('functionVersion'),
                        'max_batch_size': func.get('maxBatchSize'),
                        'runtime_name': func.get('runtime', {}).get('name'),
                        'runtime_version': func.get('runtime', {}).get('runtimeVersion'),
                    }

                    resources.append({
                        'service': 'appsync',
                        'type': 'function',
                        'id': f"{api_id}/{func_id}",
                        'arn': func_arn,
                        'name': func_name,
                        'region': region,
                        'details': details,
                        'tags': {}
                    })
        except Exception:
            pass

    # API Keys (per API)
    for api_id in api_ids:
        try:
            paginator = appsync.get_paginator('list_api_keys')
            for page in paginator.paginate(apiId=api_id):
                for key in page.get('apiKeys', []):
                    key_id = key['id']

                    details = {
                        'api_id': api_id,
                        'description': key.get('description'),
                        'expires': key.get('expires'),
                        'deletes': key.get('deletes'),
                    }

                    resources.append({
                        'service': 'appsync',
                        'type': 'api-key',
                        'id': f"{api_id}/{key_id}",
                        'arn': f"arn:aws:appsync:{region}:{account_id}:apis/{api_id}/apikeys/{key_id}",
                        'name': key.get('description', key_id),
                        'region': region,
                        'details': details,
                        'tags': {}
                    })
        except Exception:
            pass

    # Domain Names
    try:
        paginator = appsync.get_paginator('list_domain_names')
        for page in paginator.paginate():
            for domain in page.get('domainNameConfigs', []):
                domain_name = domain['domainName']

                details = {
                    'appsync_domain_name': domain.get('appsyncDomainName'),
                    'certificate_arn': domain.get('certificateArn'),
                    'description': domain.get('description'),
                    'hosted_zone_id': domain.get('hostedZoneId'),
                }

                resources.append({
                    'service': 'appsync',
                    'type': 'domain-name',
                    'id': domain_name,
                    'arn': f"arn:aws:appsync:{region}:{account_id}:domainnames/{domain_name}",
                    'name': domain_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
ECS resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_ecs_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect ECS resources: clusters, services, task definitions, capacity providers.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    ecs = session.client('ecs', region_name=region)

    # ECS Clusters
    cluster_arns = []
    try:
        paginator = ecs.get_paginator('list_clusters')
        for page in paginator.paginate():
            cluster_arns.extend(page.get('clusterArns', []))
    except Exception:
        pass

    if cluster_arns:
        try:
            # Describe clusters in batches of 100
            for i in range(0, len(cluster_arns), 100):
                batch = cluster_arns[i:i+100]
                response = ecs.describe_clusters(
                    clusters=batch,
                    include=['TAGS', 'SETTINGS', 'STATISTICS']
                )

                for cluster in response.get('clusters', []):
                    cluster_name = cluster['clusterName']
                    cluster_arn = cluster['clusterArn']

                    # Get tags from response
                    tags = {}
                    for tag in cluster.get('tags', []):
                        tags[tag.get('key', '')] = tag.get('value', '')

                    resources.append({
                        'service': 'ecs',
                        'type': 'cluster',
                        'id': cluster_name,
                        'arn': cluster_arn,
                        'name': cluster_name,
                        'region': region,
                        'details': {
                            'status': cluster.get('status'),
                            'running_tasks_count': cluster.get('runningTasksCount'),
                            'pending_tasks_count': cluster.get('pendingTasksCount'),
                            'active_services_count': cluster.get('activeServicesCount'),
                            'registered_container_instances_count': cluster.get('registeredContainerInstancesCount'),
                            'capacity_providers': cluster.get('capacityProviders', []),
                        },
                        'tags': tags
                    })

                    # ECS Services for this cluster
                    try:
                        service_arns = []
                        svc_paginator = ecs.get_paginator('list_services')
                        for svc_page in svc_paginator.paginate(cluster=cluster_arn):
                            service_arns.extend(svc_page.get('serviceArns', []))

                        if service_arns:
                            # Describe services in batches of 10
                            for j in range(0, len(service_arns), 10):
                                svc_batch = service_arns[j:j+10]
                                svc_response = ecs.describe_services(
                                    cluster=cluster_arn,
                                    services=svc_batch,
                                    include=['TAGS']
                                )

                                for svc in svc_response.get('services', []):
                                    svc_name = svc['serviceName']

                                    svc_tags = {}
                                    for tag in svc.get('tags', []):
                                        svc_tags[tag.get('key', '')] = tag.get('value', '')

                                    resources.append({
                                        'service': 'ecs',
                                        'type': 'service',
                                        'id': svc_name,
                                        'arn': svc['serviceArn'],
                                        'name': svc_name,
                                        'region': region,
                                        'details': {
                                            'cluster': cluster_name,
                                            'status': svc.get('status'),
                                            'desired_count': svc.get('desiredCount'),
                                            'running_count': svc.get('runningCount'),
                                            'pending_count': svc.get('pendingCount'),
                                            'launch_type': svc.get('launchType'),
                                            'task_definition': svc.get('taskDefinition'),
                                            'deployment_controller': svc.get('deploymentController', {}).get('type'),
                                            'scheduling_strategy': svc.get('schedulingStrategy'),
                                        },
                                        'tags': svc_tags
                                    })
                    except Exception:
                        pass
        except Exception:
            pass

    # ECS Task Definitions (only latest revisions, active)
    try:
        paginator = ecs.get_paginator('list_task_definition_families')
        for page in paginator.paginate(status='ACTIVE'):
            for family in page.get('families', []):
                try:
                    # Get latest revision
                    td_response = ecs.describe_task_definition(
                        taskDefinition=family,
                        include=['TAGS']
                    )

                    td = td_response.get('taskDefinition', {})
                    td_arn = td['taskDefinitionArn']

                    td_tags = {}
                    for tag in td_response.get('tags', []):
                        td_tags[tag.get('key', '')] = tag.get('value', '')

                    resources.append({
                        'service': 'ecs',
                        'type': 'task-definition',
                        'id': f"{family}:{td.get('revision')}",
                        'arn': td_arn,
                        'name': family,
                        'region': region,
                        'details': {
                            'family': family,
                            'revision': td.get('revision'),
                            'status': td.get('status'),
                            'requires_compatibilities': td.get('requiresCompatibilities', []),
                            'cpu': td.get('cpu'),
                            'memory': td.get('memory'),
                            'network_mode': td.get('networkMode'),
                            'container_count': len(td.get('containerDefinitions', [])),
                            'execution_role_arn': td.get('executionRoleArn'),
                            'task_role_arn': td.get('taskRoleArn'),
                        },
                        'tags': td_tags
                    })
                except Exception:
                    pass
    except Exception:
        pass

    # ECS Capacity Providers (non-default)
    try:
        paginator = ecs.get_paginator('describe_capacity_providers')
        for page in paginator.paginate():
            for cp in page.get('capacityProviders', []):
                cp_name = cp['capacityProviderArn'].split('/')[-1]

                # Skip AWS-managed capacity providers
                if cp_name in ['FARGATE', 'FARGATE_SPOT']:
                    continue

                cp_tags = {}
                for tag in cp.get('tags', []):
                    cp_tags[tag.get('key', '')] = tag.get('value', '')

                resources.append({
                    'service': 'ecs',
                    'type': 'capacity-provider',
                    'id': cp_name,
                    'arn': cp['capacityProviderArn'],
                    'name': cp_name,
                    'region': region,
                    'details': {
                        'status': cp.get('status'),
                        'auto_scaling_group_arn': cp.get('autoScalingGroupProvider', {}).get('autoScalingGroupArn'),
                        'managed_scaling': cp.get('autoScalingGroupProvider', {}).get('managedScaling', {}).get('status'),
                        'managed_termination_protection': cp.get('autoScalingGroupProvider', {}).get('managedTerminationProtection'),
                    },
                    'tags': cp_tags
                })
    except Exception:
        pass

    return resources



"""
Resource Access Manager resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_ram_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect RAM resources: resource shares.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    ram = session.client('ram', region_name=region)

    # Resource Shares (owned by this account)
    try:
        paginator = ram.get_paginator('get_resource_shares')
        for page in paginator.paginate(resourceOwner='SELF'):
            for share in page.get('resourceShares', []):
                share_arn = share['resourceShareArn']
                share_name = share.get('name', share_arn.split('/')[-1])

                # Tags are included in the response
                tags = {}
                for tag in share.get('tags', []):
                    tags[tag.get('key', '')] = tag.get('value', '')

                # Get shared resources count
                shared_resources = []
                try:
                    res_paginator = ram.get_paginator('list_resources')
                    for res_page in res_paginator.paginate(resourceOwner='SELF', resourceShareArns=[share_arn]):
                        shared_resources.extend(res_page.get('resources', []))
                except Exception:
                    pass

                # Get principals count
                principals = []
                try:
                    prin_paginator = ram.get_paginator('list_principals')
                    for prin_page in prin_paginator.paginate(resourceOwner='SELF', resourceShareArns=[share_arn]):
                        principals.extend(prin_page.get('principals', []))
                except Exception:
                    pass

                resources.append({
                    'service': 'ram',
                    'type': 'resource-share',
                    'id': share_arn.split('/')[-1],
                    'arn': share_arn,
                    'name': share_name,
                    'region': region,
                    'details': {
                        'status': share.get('status'),
                        'status_message': share.get('statusMessage'),
                        'allow_external_principals': share.get('allowExternalPrincipals'),
                        'creation_time': str(share.get('creationTime', '')),
                        'last_updated_time': str(share.get('lastUpdatedTime', '')),
                        'feature_set': share.get('featureSet'),
                        'shared_resources_count': len(shared_resources),
                        'principals_count': len(principals),
                        'resource_types': list(set(r.get('type') for r in shared_resources if r.get('type'))),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Resource Shares (shared with this account)
    try:
        paginator = ram.get_paginator('get_resource_shares')
        for page in paginator.paginate(resourceOwner='OTHER-ACCOUNTS'):
            for share in page.get('resourceShares', []):
                share_arn = share['resourceShareArn']
                share_name = share.get('name', share_arn.split('/')[-1])

                # Tags are included in the response
                tags = {}
                for tag in share.get('tags', []):
                    tags[tag.get('key', '')] = tag.get('value', '')

                resources.append({
                    'service': 'ram',
                    'type': 'resource-share-invitation',
                    'id': share_arn.split('/')[-1],
                    'arn': share_arn,
                    'name': share_name,
                    'region': region,
                    'details': {
                        'status': share.get('status'),
                        'owner_account_id': share.get('owningAccountId'),
                        'allow_external_principals': share.get('allowExternalPrincipals'),
                        'creation_time': str(share.get('creationTime', '')),
                        'last_updated_time': str(share.get('lastUpdatedTime', '')),
                        'feature_set': share.get('featureSet'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
X-Ray resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_xray_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect X-Ray resources: groups, sampling rules.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    xray = session.client('xray', region_name=region)

    # X-Ray Groups (skip AWS default group that exists in every region)
    try:
        paginator = xray.get_paginator('get_groups')
        for page in paginator.paginate():
            for group in page.get('Groups', []):
                group_name = group['GroupName']

                # Skip the AWS default group
                if group_name == 'Default':
                    continue

                group_arn = group['GroupARN']

                # Get tags
                tags = {}
                try:
                    tag_response = xray.list_tags_for_resource(ResourceARN=group_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'xray',
                    'type': 'group',
                    'id': group_name,
                    'arn': group_arn,
                    'name': group_name,
                    'region': region,
                    'details': {
                        'filter_expression': group.get('FilterExpression'),
                        'insights_configuration': group.get('InsightsConfiguration'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # X-Ray Sampling Rules (custom only)
    try:
        response = xray.get_sampling_rules()
        for rule_record in response.get('SamplingRuleRecords', []):
            rule = rule_record.get('SamplingRule', {})
            rule_name = rule.get('RuleName')

            # Skip the default rule
            if rule_name == 'Default':
                continue

            rule_arn = rule.get('RuleARN')

            # Get tags
            tags = {}
            if rule_arn:
                try:
                    tag_response = xray.list_tags_for_resource(ResourceARN=rule_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

            resources.append({
                'service': 'xray',
                'type': 'sampling-rule',
                'id': rule_name,
                'arn': rule_arn or f"arn:aws:xray:{region}:{account_id}:sampling-rule/{rule_name}",
                'name': rule_name,
                'region': region,
                'details': {
                    'priority': rule.get('Priority'),
                    'fixed_rate': rule.get('FixedRate'),
                    'reservoir_size': rule.get('ReservoirSize'),
                    'service_name': rule.get('ServiceName'),
                    'service_type': rule.get('ServiceType'),
                    'host': rule.get('Host'),
                    'http_method': rule.get('HTTPMethod'),
                    'url_path': rule.get('URLPath'),
                    'version': rule.get('Version'),
                    'created_at': str(rule_record.get('CreatedAt', '')),
                    'modified_at': str(rule_record.get('ModifiedAt', '')),
                },
                'tags': tags
            })
    except Exception:
        pass

    return resources



"""
AWS Serverless Application Repository resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# Serverless Application Repository supported regions (from https://docs.aws.amazon.com/general/latest/gr/serverlessrepo.html)
SERVERLESSREPO_REGIONS = {
    'us-east-1', 'us-east-2', 'us-west-1', 'us-west-2',
    'ap-east-1', 'ap-south-1', 'ap-northeast-1', 'ap-northeast-2', 'ap-southeast-1', 'ap-southeast-2',
    'ca-central-1',
    'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-west-3', 'eu-north-1',
    'me-south-1',
    'sa-east-1',
}


def collect_serverlessrepo_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Serverless Application Repository resources: applications.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in SERVERLESSREPO_REGIONS:
        return []

    resources = []
    sar = session.client('serverlessrepo', region_name=region)

    # Applications
    try:
        paginator = sar.get_paginator('list_applications')
        for page in paginator.paginate():
            for app in page.get('Applications', []):
                # ApplicationId is actually the ARN
                app_arn = app['ApplicationId']
                app_name = app.get('Name', app_arn.split('/')[-1])
                # Extract just the ID from the ARN
                app_id = app_arn.split('/')[-1] if '/' in app_arn else app_arn

                details = {
                    'author': app.get('Author'),
                    'description': app.get('Description'),
                    'home_page_url': app.get('HomePageUrl'),
                    'labels': app.get('Labels', []),
                    'spdx_license_id': app.get('SpdxLicenseId'),
                }

                resources.append({
                    'service': 'serverlessrepo',
                    'type': 'application',
                    'id': app_id,
                    'arn': app_arn,
                    'name': app_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Textract resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


# Textract supported regions (from https://docs.aws.amazon.com/general/latest/gr/textract.html)
TEXTRACT_REGIONS = {
    'us-east-1', 'us-east-2', 'us-west-1', 'us-west-2',
    'ap-south-1', 'ap-northeast-2', 'ap-southeast-1', 'ap-southeast-2',
    'ca-central-1',
    'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-west-3', 'eu-south-2',
}


def collect_textract_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Textract resources: adapters.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in TEXTRACT_REGIONS:
        return []

    resources = []
    textract = session.client('textract', region_name=region)

    # Adapters
    try:
        paginator = textract.get_paginator('list_adapters')
        for page in paginator.paginate():
            for adapter in page.get('Adapters', []):
                adapter_id = adapter['AdapterId']
                adapter_name = adapter.get('AdapterName', adapter_id)

                details = {
                    'creation_time': str(adapter.get('CreationTime', '')),
                    'feature_types': adapter.get('FeatureTypes', []),
                }

                # Get adapter details
                try:
                    adapter_info = textract.get_adapter(AdapterId=adapter_id)
                    details['description'] = adapter_info.get('Description')
                    details['auto_update'] = adapter_info.get('AutoUpdate')
                    tags = adapter_info.get('Tags', {})
                except Exception:
                    tags = {}

                resources.append({
                    'service': 'textract',
                    'type': 'adapter',
                    'id': adapter_id,
                    'arn': f"arn:aws:textract:{region}:{account_id}:adapter/{adapter_id}",
                    'name': adapter_name,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
Amazon Keyspaces resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_keyspaces_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Amazon Keyspaces resources: keyspaces, tables.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    keyspaces = session.client('keyspaces', region_name=region)

    # Keyspaces (skip AWS system keyspaces: system, system_schema, system_schema_mcs, system_multiregion_info)
    keyspace_list = []
    try:
        paginator = keyspaces.get_paginator('list_keyspaces')
        for page in paginator.paginate():
            for ks in page.get('keyspaces', []):
                ks_name = ks['keyspaceName']

                # Skip AWS system keyspaces
                if ks_name.startswith('system'):
                    continue

                keyspace_list.append(ks)
                ks_arn = ks['resourceArn']

                # Get tags
                tags = {}
                try:
                    tag_response = keyspaces.list_tags_for_resource(resourceArn=ks_arn)
                    for tag in tag_response.get('tags', []):
                        tags[tag.get('key', '')] = tag.get('value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'keyspaces',
                    'type': 'keyspace',
                    'id': ks_name,
                    'arn': ks_arn,
                    'name': ks_name,
                    'region': region,
                    'details': {
                        'replication_strategy': ks.get('replicationStrategy'),
                        'replication_regions': ks.get('replicationRegions', []),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Tables (for each user keyspace)
    for ks in keyspace_list:
        ks_name = ks['keyspaceName']
        try:
            table_paginator = keyspaces.get_paginator('list_tables')
            for table_page in table_paginator.paginate(keyspaceName=ks_name):
                for table in table_page.get('tables', []):
                    table_name = table['tableName']
                    table_arn = table['resourceArn']

                    try:
                        # Get table details
                        table_response = keyspaces.get_table(
                            keyspaceName=ks_name,
                            tableName=table_name
                        )

                        # Get tags
                        tags = {}
                        try:
                            tag_response = keyspaces.list_tags_for_resource(resourceArn=table_arn)
                            for tag in tag_response.get('tags', []):
                                tags[tag.get('key', '')] = tag.get('value', '')
                        except Exception:
                            pass

                        resources.append({
                            'service': 'keyspaces',
                            'type': 'table',
                            'id': f"{ks_name}/{table_name}",
                            'arn': table_arn,
                            'name': table_name,
                            'region': region,
                            'details': {
                                'keyspace_name': ks_name,
                                'status': table_response.get('status'),
                                'default_time_to_live': table_response.get('defaultTimeToLive'),
                                'creation_timestamp': str(table_response.get('creationTimestamp', '')),
                                'capacity_specification_mode': table_response.get('capacitySpecification', {}).get('throughputMode'),
                                'read_capacity_units': table_response.get('capacitySpecification', {}).get('readCapacityUnits'),
                                'write_capacity_units': table_response.get('capacitySpecification', {}).get('writeCapacityUnits'),
                                'encryption_type': table_response.get('encryptionSpecification', {}).get('type'),
                                'point_in_time_recovery_enabled': table_response.get('pointInTimeRecovery', {}).get('status') == 'ENABLED',
                                'ttl_status': table_response.get('ttl', {}).get('status'),
                            },
                            'tags': tags
                        })
                    except Exception:
                        pass
        except Exception:
            pass

    return resources



"""
AWS Firewall Manager (FMS) resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_fms_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Firewall Manager resources: policies, apps lists, protocols lists,
    and resource sets.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    fms = session.client('fms', region_name=region)

    # Check if FMS admin is configured
    try:
        fms.get_admin_account()
    except Exception:
        # Not an FMS admin account or FMS not configured
        return []

    # Policies
    try:
        paginator = fms.get_paginator('list_policies')
        for page in paginator.paginate():
            for policy in page.get('PolicyList', []):
                policy_id = policy['PolicyId']
                policy_name = policy.get('PolicyName', policy_id)
                policy_arn = policy['PolicyArn']

                details = {
                    'resource_type': policy.get('ResourceType'),
                    'security_service_type': policy.get('SecurityServiceType'),
                    'remediation_enabled': policy.get('RemediationEnabled'),
                    'delete_unused_fm_managed_resources': policy.get('DeleteUnusedFMManagedResources'),
                    'policy_status': policy.get('PolicyStatus'),
                }

                # Get full policy details
                try:
                    policy_detail = fms.get_policy(PolicyId=policy_id)
                    pol = policy_detail.get('Policy', {})
                    details['exclude_resource_tags'] = pol.get('ExcludeResourceTags')
                    details['resource_type_list'] = pol.get('ResourceTypeList', [])
                    details['resource_set_ids'] = pol.get('ResourceSetIds', [])

                    security_config = pol.get('SecurityServicePolicyData', {})
                    details['managed_service_data'] = security_config.get('ManagedServiceData')
                    details['policy_option'] = security_config.get('PolicyOption')
                except Exception:
                    pass

                resources.append({
                    'service': 'fms',
                    'type': 'policy',
                    'id': policy_id,
                    'arn': policy_arn,
                    'name': policy_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Apps Lists
    try:
        paginator = fms.get_paginator('list_apps_lists')
        for page in paginator.paginate(DefaultLists=False):
            for apps_list in page.get('AppsLists', []):
                list_id = apps_list['ListId']
                list_name = apps_list.get('ListName', list_id)
                list_arn = apps_list['ListArn']

                details = {
                    'create_time': str(apps_list.get('CreateTime', '')),
                    'last_update_time': str(apps_list.get('LastUpdateTime', '')),
                }

                # Get full list details
                try:
                    list_detail = fms.get_apps_list(ListId=list_id)
                    apps = list_detail.get('AppsList', {})
                    details['apps_count'] = len(apps.get('AppsList', []))
                    details['previous_apps_count'] = len(apps.get('PreviousAppsList', {}))
                except Exception:
                    pass

                resources.append({
                    'service': 'fms',
                    'type': 'apps-list',
                    'id': list_id,
                    'arn': list_arn,
                    'name': list_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Protocols Lists
    try:
        paginator = fms.get_paginator('list_protocols_lists')
        for page in paginator.paginate(DefaultLists=False):
            for protocols_list in page.get('ProtocolsLists', []):
                list_id = protocols_list['ListId']
                list_name = protocols_list.get('ListName', list_id)
                list_arn = protocols_list['ListArn']

                details = {
                    'create_time': str(protocols_list.get('CreateTime', '')),
                    'last_update_time': str(protocols_list.get('LastUpdateTime', '')),
                }

                # Get full list details
                try:
                    list_detail = fms.get_protocols_list(ListId=list_id)
                    protocols = list_detail.get('ProtocolsList', {})
                    details['protocols'] = protocols.get('ProtocolsList', [])
                    details['protocols_count'] = len(protocols.get('ProtocolsList', []))
                except Exception:
                    pass

                resources.append({
                    'service': 'fms',
                    'type': 'protocols-list',
                    'id': list_id,
                    'arn': list_arn,
                    'name': list_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Resource Sets
    try:
        paginator = fms.get_paginator('list_resource_sets')
        for page in paginator.paginate():
            for resource_set in page.get('ResourceSets', []):
                rs_id = resource_set['Id']
                rs_name = resource_set.get('Name', rs_id)

                details = {
                    'description': resource_set.get('Description'),
                    'resource_type_list': resource_set.get('ResourceTypeList', []),
                    'last_update_time': str(resource_set.get('LastUpdateTime', '')),
                }

                # Get full resource set details
                try:
                    rs_detail = fms.get_resource_set(Identifier=rs_id)
                    rs = rs_detail.get('ResourceSet', {})
                    details['update_token'] = rs.get('UpdateToken')
                except Exception:
                    pass

                # Construct ARN
                rs_arn = f"arn:aws:fms:{region}:{account_id}:resource-set/{rs_id}"

                resources.append({
                    'service': 'fms',
                    'type': 'resource-set',
                    'id': rs_id,
                    'arn': rs_arn,
                    'name': rs_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS AppConfig resource collector.
"""

import boto3  # noqa: F401
from typing import List, Dict, Any, Optional


def collect_appconfig_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS AppConfig resources: applications, deployment strategies, and extensions.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    appconfig = session.client('appconfig', region_name=region)

    # Applications
    try:
        paginator = appconfig.get_paginator('list_applications')
        for page in paginator.paginate():
            for app in page.get('Items', []):
                app_id = app['Id']
                app_name = app.get('Name', app_id)
                app_arn = f"arn:aws:appconfig:{region}:{account_id}:application/{app_id}"

                details = {
                    'description': app.get('Description'),
                }

                resources.append({
                    'service': 'appconfig',
                    'type': 'application',
                    'id': app_id,
                    'arn': app_arn,
                    'name': app_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })

                # Environments for this application
                try:
                    env_paginator = appconfig.get_paginator('list_environments')
                    for env_page in env_paginator.paginate(ApplicationId=app_id):
                        for env in env_page.get('Items', []):
                            env_id = env['Id']
                            env_name = env.get('Name', env_id)
                            env_arn = f"arn:aws:appconfig:{region}:{account_id}:application/{app_id}/environment/{env_id}"

                            env_details = {
                                'description': env.get('Description'),
                                'state': env.get('State'),
                                'application_id': app_id,
                            }

                            resources.append({
                                'service': 'appconfig',
                                'type': 'environment',
                                'id': env_id,
                                'arn': env_arn,
                                'name': env_name,
                                'region': region,
                                'details': env_details,
                                'tags': {}
                            })
                except Exception:
                    pass

                # Configuration Profiles for this application
                try:
                    profile_paginator = appconfig.get_paginator('list_configuration_profiles')
                    for profile_page in profile_paginator.paginate(ApplicationId=app_id):
                        for profile in profile_page.get('Items', []):
                            profile_id = profile['Id']
                            profile_name = profile.get('Name', profile_id)
                            profile_arn = f"arn:aws:appconfig:{region}:{account_id}:application/{app_id}/configurationprofile/{profile_id}"

                            profile_details = {
                                'description': profile.get('Description'),
                                'location_uri': profile.get('LocationUri'),
                                'type': profile.get('Type'),
                                'application_id': app_id,
                            }

                            resources.append({
                                'service': 'appconfig',
                                'type': 'configuration-profile',
                                'id': profile_id,
                                'arn': profile_arn,
                                'name': profile_name,
                                'region': region,
                                'details': profile_details,
                                'tags': {}
                            })
                except Exception:
                    pass
    except Exception:
        pass

    # Deployment Strategies
    try:
        paginator = appconfig.get_paginator('list_deployment_strategies')
        for page in paginator.paginate():
            for strategy in page.get('Items', []):
                strategy_id = strategy['Id']
                strategy_name = strategy.get('Name', strategy_id)
                strategy_arn = f"arn:aws:appconfig:{region}:{account_id}:deploymentstrategy/{strategy_id}"

                # Skip predefined strategies
                if strategy_id.startswith('AppConfig.'):
                    continue

                details = {
                    'description': strategy.get('Description'),
                    'deployment_duration_in_minutes': strategy.get('DeploymentDurationInMinutes'),
                    'growth_factor': strategy.get('GrowthFactor'),
                    'growth_type': strategy.get('GrowthType'),
                    'final_bake_time_in_minutes': strategy.get('FinalBakeTimeInMinutes'),
                    'replicate_to': strategy.get('ReplicateTo'),
                }

                resources.append({
                    'service': 'appconfig',
                    'type': 'deployment-strategy',
                    'id': strategy_id,
                    'arn': strategy_arn,
                    'name': strategy_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Backup resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_backup_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Backup resources: vaults, plans, frameworks, report plans,
    restore testing plans, and Backup Gateway resources.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    backup = session.client('backup', region_name=region)

    # Backup Vaults
    try:
        paginator = backup.get_paginator('list_backup_vaults')
        for page in paginator.paginate():
            for vault in page.get('BackupVaultList', []):
                vault_name = vault['BackupVaultName']
                vault_arn = vault['BackupVaultArn']

                # Skip AWS managed vaults
                if vault_name.startswith('aws/'):
                    continue

                # Get tags
                tags = {}
                try:
                    tag_response = backup.list_tags(ResourceArn=vault_arn)
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'backup',
                    'type': 'vault',
                    'id': vault_name,
                    'arn': vault_arn,
                    'name': vault_name,
                    'region': region,
                    'details': {
                        'recovery_points': vault.get('NumberOfRecoveryPoints'),
                        'encryption_key_arn': vault.get('EncryptionKeyArn'),
                        'creator_request_id': vault.get('CreatorRequestId'),
                        'locked': vault.get('Locked'),
                        'min_retention_days': vault.get('MinRetentionDays'),
                        'max_retention_days': vault.get('MaxRetentionDays'),
                        'lock_date': str(vault.get('LockDate', '')) if vault.get('LockDate') else None,
                        'creation_date': str(vault.get('CreationDate', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Backup Plans
    try:
        paginator = backup.get_paginator('list_backup_plans')
        for page in paginator.paginate():
            for plan in page.get('BackupPlansList', []):
                plan_id = plan['BackupPlanId']
                plan_arn = plan['BackupPlanArn']
                plan_name = plan.get('BackupPlanName', plan_id)

                # Get selections count
                selections_count = 0
                try:
                    sel_response = backup.list_backup_selections(BackupPlanId=plan_id)
                    selections_count = len(sel_response.get('BackupSelectionsList', []))
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = backup.list_tags(ResourceArn=plan_arn)
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'backup',
                    'type': 'plan',
                    'id': plan_id,
                    'arn': plan_arn,
                    'name': plan_name,
                    'region': region,
                    'details': {
                        'version_id': plan.get('VersionId'),
                        'selections_count': selections_count,
                        'creator_request_id': plan.get('CreatorRequestId'),
                        'creation_date': str(plan.get('CreationDate', '')),
                        'last_execution_date': str(plan.get('LastExecutionDate', '')) if plan.get('LastExecutionDate') else None,
                        'advanced_backup_settings': plan.get('AdvancedBackupSettings'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Frameworks (Compliance)
    try:
        paginator = backup.get_paginator('list_frameworks')
        for page in paginator.paginate():
            for framework in page.get('Frameworks', []):
                framework_name = framework['FrameworkName']
                framework_arn = framework['FrameworkArn']

                # Get tags
                tags = {}
                try:
                    tag_response = backup.list_tags(ResourceArn=framework_arn)
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'backup',
                    'type': 'framework',
                    'id': framework_name,
                    'arn': framework_arn,
                    'name': framework_name,
                    'region': region,
                    'details': {
                        'description': framework.get('FrameworkDescription'),
                        'number_of_controls': framework.get('NumberOfControls'),
                        'deployment_status': framework.get('DeploymentStatus'),
                        'creation_time': str(framework.get('CreationTime', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Report Plans
    try:
        paginator = backup.get_paginator('list_report_plans')
        for page in paginator.paginate():
            for report in page.get('ReportPlans', []):
                report_name = report['ReportPlanName']
                report_arn = report['ReportPlanArn']

                # Get tags
                tags = {}
                try:
                    tag_response = backup.list_tags(ResourceArn=report_arn)
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'backup',
                    'type': 'report-plan',
                    'id': report_name,
                    'arn': report_arn,
                    'name': report_name,
                    'region': region,
                    'details': {
                        'description': report.get('ReportPlanDescription'),
                        'report_template': report.get('ReportSetting', {}).get('ReportTemplate'),
                        'last_attempted_execution_time': str(report.get('LastAttemptedExecutionTime', '')) if report.get('LastAttemptedExecutionTime') else None,
                        'last_successful_execution_time': str(report.get('LastSuccessfulExecutionTime', '')) if report.get('LastSuccessfulExecutionTime') else None,
                        'creation_time': str(report.get('CreationTime', '')),
                        'deployment_status': report.get('DeploymentStatus'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Restore Testing Plans
    try:
        paginator = backup.get_paginator('list_restore_testing_plans')
        for page in paginator.paginate():
            for plan in page.get('RestoreTestingPlans', []):
                plan_name = plan['RestoreTestingPlanName']
                plan_arn = plan['RestoreTestingPlanArn']

                # Get tags
                tags = {}
                try:
                    tag_response = backup.list_tags(ResourceArn=plan_arn)
                    tags = tag_response.get('Tags', {})
                except Exception:
                    pass

                resources.append({
                    'service': 'backup',
                    'type': 'restore-testing-plan',
                    'id': plan_name,
                    'arn': plan_arn,
                    'name': plan_name,
                    'region': region,
                    'details': {
                        'schedule_expression': plan.get('ScheduleExpression'),
                        'schedule_expression_timezone': plan.get('ScheduleExpressionTimezone'),
                        'start_window_hours': plan.get('StartWindowHours'),
                        'creation_time': str(plan.get('CreationTime', '')),
                        'last_execution_time': str(plan.get('LastExecutionTime', '')) if plan.get('LastExecutionTime') else None,
                        'last_update_time': str(plan.get('LastUpdateTime', '')) if plan.get('LastUpdateTime') else None,
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Note: Backup Gateway resources (gateways, hypervisors, virtual machines) skipped
    # for performance. They are on-premises resources rarely used and add ~16s overhead.

    return resources



"""
Redshift resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_redshift_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Redshift resources: clusters, cluster snapshots, parameter groups, subnet groups.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    redshift = session.client('redshift', region_name=region)

    # Redshift Clusters
    try:
        paginator = redshift.get_paginator('describe_clusters')
        for page in paginator.paginate():
            for cluster in page.get('Clusters', []):
                cluster_id = cluster['ClusterIdentifier']

                # Tags are included in the response
                tags = {}
                for tag in cluster.get('Tags', []):
                    tags[tag.get('Key', '')] = tag.get('Value', '')

                resources.append({
                    'service': 'redshift',
                    'type': 'cluster',
                    'id': cluster_id,
                    'arn': f"arn:aws:redshift:{region}:{account_id}:cluster:{cluster_id}",
                    'name': cluster_id,
                    'region': region,
                    'details': {
                        'node_type': cluster.get('NodeType'),
                        'cluster_status': cluster.get('ClusterStatus'),
                        'modify_status': cluster.get('ModifyStatus'),
                        'master_username': cluster.get('MasterUsername'),
                        'db_name': cluster.get('DBName'),
                        'endpoint': cluster.get('Endpoint', {}).get('Address'),
                        'port': cluster.get('Endpoint', {}).get('Port'),
                        'cluster_create_time': str(cluster.get('ClusterCreateTime', '')),
                        'automated_snapshot_retention_period': cluster.get('AutomatedSnapshotRetentionPeriod'),
                        'number_of_nodes': cluster.get('NumberOfNodes'),
                        'publicly_accessible': cluster.get('PubliclyAccessible'),
                        'encrypted': cluster.get('Encrypted'),
                        'vpc_id': cluster.get('VpcId'),
                        'cluster_version': cluster.get('ClusterVersion'),
                        'enhanced_vpc_routing': cluster.get('EnhancedVpcRouting'),
                        'maintenance_track_name': cluster.get('MaintenanceTrackName'),
                        'elastic_resize_number_of_node_options': cluster.get('ElasticResizeNumberOfNodeOptions'),
                        'total_storage_capacity_in_mega_bytes': cluster.get('TotalStorageCapacityInMegaBytes'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Serverless Workgroups
    try:
        redshift_serverless = session.client('redshift-serverless', region_name=region)
        paginator = redshift_serverless.get_paginator('list_workgroups')
        for page in paginator.paginate():
            for wg in page.get('workgroups', []):
                wg_name = wg['workgroupName']
                wg_arn = wg['workgroupArn']

                # Get tags
                tags = {}
                try:
                    tag_response = redshift_serverless.list_tags_for_resource(resourceArn=wg_arn)
                    for tag in tag_response.get('tags', []):
                        tags[tag.get('key', '')] = tag.get('value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'redshift',
                    'type': 'serverless-workgroup',
                    'id': wg['workgroupId'],
                    'arn': wg_arn,
                    'name': wg_name,
                    'region': region,
                    'details': {
                        'status': wg.get('status'),
                        'namespace_name': wg.get('namespaceName'),
                        'base_capacity': wg.get('baseCapacity'),
                        'enhanced_vpc_routing': wg.get('enhancedVpcRouting'),
                        'publicly_accessible': wg.get('publiclyAccessible'),
                        'endpoint': wg.get('endpoint', {}).get('address'),
                        'port': wg.get('port'),
                        'creation_date': str(wg.get('creationDate', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Serverless Namespaces
    try:
        redshift_serverless = session.client('redshift-serverless', region_name=region)
        paginator = redshift_serverless.get_paginator('list_namespaces')
        for page in paginator.paginate():
            for ns in page.get('namespaces', []):
                ns_name = ns['namespaceName']
                ns_arn = ns['namespaceArn']

                # Get tags
                tags = {}
                try:
                    tag_response = redshift_serverless.list_tags_for_resource(resourceArn=ns_arn)
                    for tag in tag_response.get('tags', []):
                        tags[tag.get('key', '')] = tag.get('value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'redshift',
                    'type': 'serverless-namespace',
                    'id': ns['namespaceId'],
                    'arn': ns_arn,
                    'name': ns_name,
                    'region': region,
                    'details': {
                        'status': ns.get('status'),
                        'admin_username': ns.get('adminUsername'),
                        'db_name': ns.get('dbName'),
                        'kms_key_id': ns.get('kmsKeyId'),
                        'default_iam_role_arn': ns.get('defaultIamRoleArn'),
                        'creation_date': str(ns.get('creationDate', '')),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Custom Parameter Groups (not default)
    try:
        paginator = redshift.get_paginator('describe_cluster_parameter_groups')
        for page in paginator.paginate():
            for pg in page.get('ParameterGroups', []):
                pg_name = pg['ParameterGroupName']

                # Skip default parameter groups
                if pg_name.startswith('default.'):
                    continue

                # Tags are included in the response
                tags = {}
                for tag in pg.get('Tags', []):
                    tags[tag.get('Key', '')] = tag.get('Value', '')

                resources.append({
                    'service': 'redshift',
                    'type': 'parameter-group',
                    'id': pg_name,
                    'arn': f"arn:aws:redshift:{region}:{account_id}:parametergroup:{pg_name}",
                    'name': pg_name,
                    'region': region,
                    'details': {
                        'parameter_group_family': pg.get('ParameterGroupFamily'),
                        'description': pg.get('Description'),
                    },
                    'tags': tags
                })
    except Exception:
        pass

    # Subnet Groups
    try:
        paginator = redshift.get_paginator('describe_cluster_subnet_groups')
        for page in paginator.paginate():
            for sg in page.get('ClusterSubnetGroups', []):
                sg_name = sg['ClusterSubnetGroupName']

                # Tags are included in the response
                tags = {}
                for tag in sg.get('Tags', []):
                    tags[tag.get('Key', '')] = tag.get('Value', '')

                resources.append({
                    'service': 'redshift',
                    'type': 'subnet-group',
                    'id': sg_name,
                    'arn': f"arn:aws:redshift:{region}:{account_id}:subnetgroup:{sg_name}",
                    'name': sg_name,
                    'region': region,
                    'details': {
                        'description': sg.get('Description'),
                        'vpc_id': sg.get('VpcId'),
                        'subnet_group_status': sg.get('SubnetGroupStatus'),
                        'subnets': [s.get('SubnetIdentifier') for s in sg.get('Subnets', [])],
                    },
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
AWS Transfer Family resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_transfer_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Transfer Family resources: servers, users, workflows, connectors,
    certificates, profiles, agreements, host keys, and web apps.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    transfer = session.client('transfer', region_name=region)

    # Servers
    server_ids = []
    try:
        paginator = transfer.get_paginator('list_servers')
        for page in paginator.paginate():
            for server in page.get('Servers', []):
                server_id = server['ServerId']
                server_ids.append(server_id)
                server_arn = server.get('Arn', f"arn:aws:transfer:{region}:{account_id}:server/{server_id}")

                details = {
                    'state': server.get('State'),
                    'endpoint_type': server.get('EndpointType'),
                    'identity_provider_type': server.get('IdentityProviderType'),
                    'logging_role': server.get('LoggingRole'),
                    'user_count': server.get('UserCount'),
                    'domain': server.get('Domain'),
                }

                # Get detailed server info
                try:
                    desc_response = transfer.describe_server(ServerId=server_id)
                    srv = desc_response.get('Server', {})
                    details.update({
                        'protocols': srv.get('Protocols', []),
                        'security_policy_name': srv.get('SecurityPolicyName'),
                        'certificate': srv.get('Certificate'),
                        'endpoint': srv.get('EndpointDetails', {}).get('AddressAllocationIds'),
                        'vpc_endpoint_id': srv.get('EndpointDetails', {}).get('VpcEndpointId'),
                        'vpc_id': srv.get('EndpointDetails', {}).get('VpcId'),
                        'subnet_ids': srv.get('EndpointDetails', {}).get('SubnetIds'),
                        'security_group_ids': srv.get('EndpointDetails', {}).get('SecurityGroupIds'),
                        'host_key_fingerprint': srv.get('HostKeyFingerprint'),
                        'workflow_on_upload': srv.get('WorkflowDetails', {}).get('OnUpload'),
                        'workflow_on_partial_upload': srv.get('WorkflowDetails', {}).get('OnPartialUpload'),
                        'structured_log_destinations': srv.get('StructuredLogDestinations'),
                        's3_storage_options': srv.get('S3StorageOptions'),
                    })
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = transfer.list_tags_for_resource(Arn=server_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'transfer',
                    'type': 'server',
                    'id': server_id,
                    'arn': server_arn,
                    'name': server_id,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Users (per server)
    for server_id in server_ids:
        try:
            paginator = transfer.get_paginator('list_users')
            for page in paginator.paginate(ServerId=server_id):
                for user in page.get('Users', []):
                    user_name = user['UserName']
                    user_arn = user.get('Arn', f"arn:aws:transfer:{region}:{account_id}:user/{server_id}/{user_name}")

                    details = {
                        'server_id': server_id,
                        'home_directory_type': user.get('HomeDirectoryType'),
                        'home_directory': user.get('HomeDirectory'),
                        'role': user.get('Role'),
                        'ssh_public_key_count': user.get('SshPublicKeyCount'),
                    }

                    # Get detailed user info
                    try:
                        desc_response = transfer.describe_user(ServerId=server_id, UserName=user_name)
                        usr = desc_response.get('User', {})
                        details.update({
                            'home_directory_mappings': usr.get('HomeDirectoryMappings'),
                            'policy': usr.get('Policy'),
                            'posix_profile': usr.get('PosixProfile'),
                        })
                    except Exception:
                        pass

                    # Get tags
                    tags = {}
                    try:
                        tag_response = transfer.list_tags_for_resource(Arn=user_arn)
                        for tag in tag_response.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                    except Exception:
                        pass

                    resources.append({
                        'service': 'transfer',
                        'type': 'user',
                        'id': f"{server_id}/{user_name}",
                        'arn': user_arn,
                        'name': user_name,
                        'region': region,
                        'details': details,
                        'tags': tags
                    })
        except Exception:
            pass

    # Host Keys (per server)
    for server_id in server_ids:
        try:
            paginator = transfer.get_paginator('list_host_keys')
            for page in paginator.paginate(ServerId=server_id):
                for key in page.get('HostKeys', []):
                    key_id = key['HostKeyId']
                    key_arn = key.get('Arn', f"arn:aws:transfer:{region}:{account_id}:host-key/{server_id}/{key_id}")

                    # Get tags
                    tags = {}
                    try:
                        tag_response = transfer.list_tags_for_resource(Arn=key_arn)
                        for tag in tag_response.get('Tags', []):
                            tags[tag.get('Key', '')] = tag.get('Value', '')
                    except Exception:
                        pass

                    resources.append({
                        'service': 'transfer',
                        'type': 'host-key',
                        'id': f"{server_id}/{key_id}",
                        'arn': key_arn,
                        'name': key.get('Description', key_id),
                        'region': region,
                        'details': {
                            'server_id': server_id,
                            'fingerprint': key.get('Fingerprint'),
                            'type': key.get('Type'),
                            'date_imported': str(key.get('DateImported', '')),
                        },
                        'tags': tags
                    })
        except Exception:
            pass

    # Workflows
    try:
        paginator = transfer.get_paginator('list_workflows')
        for page in paginator.paginate():
            for workflow in page.get('Workflows', []):
                workflow_id = workflow['WorkflowId']
                workflow_arn = workflow.get('Arn', f"arn:aws:transfer:{region}:{account_id}:workflow/{workflow_id}")

                details = {
                    'description': workflow.get('Description'),
                }

                # Get detailed workflow info
                try:
                    desc_response = transfer.describe_workflow(WorkflowId=workflow_id)
                    wf = desc_response.get('Workflow', {})
                    steps = wf.get('Steps', [])
                    on_exception_steps = wf.get('OnExceptionSteps', [])
                    details.update({
                        'steps_count': len(steps),
                        'on_exception_steps_count': len(on_exception_steps),
                        'step_types': [s.get('Type') for s in steps],
                    })
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = transfer.list_tags_for_resource(Arn=workflow_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'transfer',
                    'type': 'workflow',
                    'id': workflow_id,
                    'arn': workflow_arn,
                    'name': workflow.get('Description', workflow_id),
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Connectors
    try:
        paginator = transfer.get_paginator('list_connectors')
        for page in paginator.paginate():
            for connector in page.get('Connectors', []):
                connector_id = connector['ConnectorId']
                connector_arn = connector.get('Arn', f"arn:aws:transfer:{region}:{account_id}:connector/{connector_id}")

                details = {
                    'url': connector.get('Url'),
                }

                # Get detailed connector info
                try:
                    desc_response = transfer.describe_connector(ConnectorId=connector_id)
                    conn = desc_response.get('Connector', {})
                    details.update({
                        'access_role': conn.get('AccessRole'),
                        'logging_role': conn.get('LoggingRole'),
                        'security_policy_name': conn.get('SecurityPolicyName'),
                        'sftp_user_secret_id': conn.get('SftpConfig', {}).get('UserSecretId'),
                        'sftp_trusted_host_keys': conn.get('SftpConfig', {}).get('TrustedHostKeys'),
                        'as2_local_profile_id': conn.get('As2Config', {}).get('LocalProfileId'),
                        'as2_partner_profile_id': conn.get('As2Config', {}).get('PartnerProfileId'),
                    })
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = transfer.list_tags_for_resource(Arn=connector_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'transfer',
                    'type': 'connector',
                    'id': connector_id,
                    'arn': connector_arn,
                    'name': connector_id,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Certificates
    try:
        paginator = transfer.get_paginator('list_certificates')
        for page in paginator.paginate():
            for cert in page.get('Certificates', []):
                cert_id = cert['CertificateId']
                cert_arn = cert.get('Arn', f"arn:aws:transfer:{region}:{account_id}:certificate/{cert_id}")

                details = {
                    'usage': cert.get('Usage'),
                    'status': cert.get('Status'),
                    'active_date': str(cert.get('ActiveDate', '')),
                    'inactive_date': str(cert.get('InactiveDate', '')),
                    'type': cert.get('Type'),
                    'description': cert.get('Description'),
                }

                # Get detailed certificate info
                try:
                    desc_response = transfer.describe_certificate(CertificateId=cert_id)
                    certificate = desc_response.get('Certificate', {})
                    details.update({
                        'serial': certificate.get('Serial'),
                        'not_before_date': str(certificate.get('NotBeforeDate', '')),
                        'not_after_date': str(certificate.get('NotAfterDate', '')),
                    })
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = transfer.list_tags_for_resource(Arn=cert_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'transfer',
                    'type': 'certificate',
                    'id': cert_id,
                    'arn': cert_arn,
                    'name': cert.get('Description', cert_id),
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Profiles (AS2)
    try:
        paginator = transfer.get_paginator('list_profiles')
        for page in paginator.paginate():
            for profile in page.get('Profiles', []):
                profile_id = profile['ProfileId']
                profile_arn = profile.get('Arn', f"arn:aws:transfer:{region}:{account_id}:profile/{profile_id}")

                details = {
                    'profile_type': profile.get('ProfileType'),
                    'as2_id': profile.get('As2Id'),
                }

                # Get detailed profile info
                try:
                    desc_response = transfer.describe_profile(ProfileId=profile_id)
                    prof = desc_response.get('Profile', {})
                    details.update({
                        'certificate_ids': prof.get('CertificateIds', []),
                    })
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = transfer.list_tags_for_resource(Arn=profile_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'transfer',
                    'type': 'profile',
                    'id': profile_id,
                    'arn': profile_arn,
                    'name': profile.get('As2Id', profile_id),
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # Agreements (AS2)
    try:
        paginator = transfer.get_paginator('list_agreements')
        # Agreements require a server ID, so iterate over servers
        for server_id in server_ids:
            try:
                for page in paginator.paginate(ServerId=server_id):
                    for agreement in page.get('Agreements', []):
                        agreement_id = agreement['AgreementId']
                        agreement_arn = agreement.get('Arn', f"arn:aws:transfer:{region}:{account_id}:agreement/{server_id}/{agreement_id}")

                        details = {
                            'server_id': server_id,
                            'status': agreement.get('Status'),
                            'local_profile_id': agreement.get('LocalProfileId'),
                            'partner_profile_id': agreement.get('PartnerProfileId'),
                        }

                        # Get detailed agreement info
                        try:
                            desc_response = transfer.describe_agreement(AgreementId=agreement_id, ServerId=server_id)
                            agr = desc_response.get('Agreement', {})
                            details.update({
                                'description': agr.get('Description'),
                                'base_directory': agr.get('BaseDirectory'),
                                'access_role': agr.get('AccessRole'),
                            })
                        except Exception:
                            pass

                        # Get tags
                        tags = {}
                        try:
                            tag_response = transfer.list_tags_for_resource(Arn=agreement_arn)
                            for tag in tag_response.get('Tags', []):
                                tags[tag.get('Key', '')] = tag.get('Value', '')
                        except Exception:
                            pass

                        resources.append({
                            'service': 'transfer',
                            'type': 'agreement',
                            'id': f"{server_id}/{agreement_id}",
                            'arn': agreement_arn,
                            'name': agreement.get('Description', agreement_id),
                            'region': region,
                            'details': details,
                            'tags': tags
                        })
            except Exception:
                pass
    except Exception:
        pass

    # Web Apps
    try:
        paginator = transfer.get_paginator('list_web_apps')
        for page in paginator.paginate():
            for webapp in page.get('WebApps', []):
                webapp_id = webapp['WebAppId']
                webapp_arn = webapp.get('Arn', f"arn:aws:transfer:{region}:{account_id}:webapp/{webapp_id}")

                details = {
                    'access_endpoint': webapp.get('AccessEndpoint'),
                    'web_app_endpoint': webapp.get('WebAppEndpoint'),
                }

                # Get detailed web app info
                try:
                    desc_response = transfer.describe_web_app(WebAppId=webapp_id)
                    app = desc_response.get('WebApp', {})
                    details.update({
                        'identity_provider_type': app.get('IdentityProviderDetails', {}).get('IdentityProviderType'),
                        'described_identity_provider_details': app.get('DescribedIdentityProviderDetails'),
                    })
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = transfer.list_tags_for_resource(Arn=webapp_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'transfer',
                    'type': 'web-app',
                    'id': webapp_id,
                    'arn': webapp_arn,
                    'name': webapp_id,
                    'region': region,
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    return resources



"""
Direct Connect resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_directconnect_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Direct Connect resources: connections, virtual interfaces, gateways, LAGs.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    dx = session.client('directconnect', region_name=region)

    # Direct Connect Connections
    try:
        response = dx.describe_connections()
        for conn in response.get('connections', []):
            conn_id = conn['connectionId']

            # Get tags
            tags = {}
            try:
                tag_response = dx.describe_tags(resourceArns=[conn_id])
                for tag_res in tag_response.get('resourceTags', []):
                    for tag in tag_res.get('tags', []):
                        tags[tag.get('key', '')] = tag.get('value', '')
            except Exception:
                pass

            resources.append({
                'service': 'directconnect',
                'type': 'connection',
                'id': conn_id,
                'arn': f"arn:aws:directconnect:{region}:{account_id}:dxcon/{conn_id}",
                'name': conn.get('connectionName', conn_id),
                'region': region,
                'details': {
                    'connection_state': conn.get('connectionState'),
                    'location': conn.get('location'),
                    'bandwidth': conn.get('bandwidth'),
                    'vlan': conn.get('vlan'),
                    'partner_name': conn.get('partnerName'),
                    'loa_issue_time': str(conn.get('loaIssueTime', '')),
                    'lag_id': conn.get('lagId'),
                    'aws_device': conn.get('awsDevice'),
                    'aws_device_v2': conn.get('awsDeviceV2'),
                    'aws_logical_device_id': conn.get('awsLogicalDeviceId'),
                    'has_logical_redundancy': conn.get('hasLogicalRedundancy'),
                    'jumbo_frame_capable': conn.get('jumboFrameCapable'),
                    'port_encryption_status': conn.get('portEncryptionStatus'),
                    'mac_sec_capable': conn.get('macSecCapable'),
                    'encryption_mode': conn.get('encryptionMode'),
                    'provider_name': conn.get('providerName'),
                },
                'tags': tags
            })
    except Exception:
        pass

    # Direct Connect Virtual Interfaces
    try:
        response = dx.describe_virtual_interfaces()
        for vif in response.get('virtualInterfaces', []):
            vif_id = vif['virtualInterfaceId']

            # Get tags
            tags = {}
            try:
                tag_response = dx.describe_tags(resourceArns=[vif_id])
                for tag_res in tag_response.get('resourceTags', []):
                    for tag in tag_res.get('tags', []):
                        tags[tag.get('key', '')] = tag.get('value', '')
            except Exception:
                pass

            resources.append({
                'service': 'directconnect',
                'type': f"virtual-interface-{vif.get('virtualInterfaceType', 'private').lower()}",
                'id': vif_id,
                'arn': f"arn:aws:directconnect:{region}:{account_id}:dxvif/{vif_id}",
                'name': vif.get('virtualInterfaceName', vif_id),
                'region': region,
                'details': {
                    'virtual_interface_state': vif.get('virtualInterfaceState'),
                    'virtual_interface_type': vif.get('virtualInterfaceType'),
                    'connection_id': vif.get('connectionId'),
                    'vlan': vif.get('vlan'),
                    'asn': vif.get('asn'),
                    'amazon_side_asn': vif.get('amazonSideAsn'),
                    'auth_key': vif.get('authKey'),
                    'amazon_address': vif.get('amazonAddress'),
                    'customer_address': vif.get('customerAddress'),
                    'address_family': vif.get('addressFamily'),
                    'virtual_gateway_id': vif.get('virtualGatewayId'),
                    'direct_connect_gateway_id': vif.get('directConnectGatewayId'),
                    'route_filter_prefixes': vif.get('routeFilterPrefixes', []),
                    'bgp_peers': len(vif.get('bgpPeers', [])),
                    'mtu': vif.get('mtu'),
                    'jumbo_frame_capable': vif.get('jumboFrameCapable'),
                    'site_link_enabled': vif.get('siteLinkEnabled'),
                },
                'tags': tags
            })
    except Exception:
        pass

    # Direct Connect Gateways
    try:
        response = dx.describe_direct_connect_gateways()
        for gw in response.get('directConnectGateways', []):
            gw_id = gw['directConnectGatewayId']

            resources.append({
                'service': 'directconnect',
                'type': 'gateway',
                'id': gw_id,
                'arn': f"arn:aws:directconnect::{account_id}:dx-gateway/{gw_id}",
                'name': gw.get('directConnectGatewayName', gw_id),
                'region': 'global',
                'details': {
                    'state': gw.get('directConnectGatewayState'),
                    'amazon_side_asn': gw.get('amazonSideAsn'),
                    'owner_account': gw.get('ownerAccount'),
                    'state_change_error': gw.get('stateChangeError'),
                },
                'tags': {}
            })
    except Exception:
        pass

    # LAGs (Link Aggregation Groups)
    try:
        response = dx.describe_lags()
        for lag in response.get('lags', []):
            lag_id = lag['lagId']

            # Get tags
            tags = {}
            try:
                tag_response = dx.describe_tags(resourceArns=[lag_id])
                for tag_res in tag_response.get('resourceTags', []):
                    for tag in tag_res.get('tags', []):
                        tags[tag.get('key', '')] = tag.get('value', '')
            except Exception:
                pass

            resources.append({
                'service': 'directconnect',
                'type': 'lag',
                'id': lag_id,
                'arn': f"arn:aws:directconnect:{region}:{account_id}:dxlag/{lag_id}",
                'name': lag.get('lagName', lag_id),
                'region': region,
                'details': {
                    'lag_state': lag.get('lagState'),
                    'location': lag.get('location'),
                    'minimum_links': lag.get('minimumLinks'),
                    'number_of_connections': lag.get('numberOfConnections'),
                    'connections_bandwidth': lag.get('connectionsBandwidth'),
                    'aws_device': lag.get('awsDevice'),
                    'aws_device_v2': lag.get('awsDeviceV2'),
                    'aws_logical_device_id': lag.get('awsLogicalDeviceId'),
                    'has_logical_redundancy': lag.get('hasLogicalRedundancy'),
                    'jumbo_frame_capable': lag.get('jumboFrameCapable'),
                    'allows_hosted_connections': lag.get('allowsHostedConnections'),
                    'encryption_mode': lag.get('encryptionMode'),
                    'mac_sec_capable': lag.get('macSecCapable'),
                    'provider_name': lag.get('providerName'),
                },
                'tags': tags
            })
    except Exception:
        pass

    return resources



"""
AWS Elemental MediaStore resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


# MediaStore supported regions (from https://docs.aws.amazon.com/general/latest/gr/mediastore.html)
MEDIASTORE_REGIONS = {
    'us-east-1', 'us-west-2',
    'ap-northeast-1', 'ap-northeast-2', 'ap-southeast-2',
    'eu-central-1', 'eu-west-1', 'eu-west-2', 'eu-north-1',
}


def collect_mediastore_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Elemental MediaStore resources: containers.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    # Skip unsupported regions
    if region not in MEDIASTORE_REGIONS:
        return []

    resources = []
    mediastore = session.client('mediastore', region_name=region)

    # Containers
    try:
        paginator = mediastore.get_paginator('list_containers')
        for page in paginator.paginate():
            for container in page.get('Containers', []):
                container_arn = container.get('ARN', '')
                container_name = container.get('Name', container_arn.split('/')[-1])

                details = {
                    'status': container.get('Status'),
                    'endpoint': container.get('Endpoint'),
                    'access_logging_enabled': container.get('AccessLoggingEnabled'),
                }

                resources.append({
                    'service': 'mediastore',
                    'type': 'container',
                    'id': container_name,
                    'arn': container_arn,
                    'name': container_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Route 53 Resolver resource collector.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_route53resolver_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect Route 53 Resolver resources: endpoints, rules, query log configs, firewall rule groups.

    Args:
        session: boto3.Session to use
        region: AWS region
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    resolver = session.client('route53resolver', region_name=region)

    # Resolver Endpoints
    try:
        paginator = resolver.get_paginator('list_resolver_endpoints')
        for page in paginator.paginate():
            for endpoint in page.get('ResolverEndpoints', []):
                endpoint_id = endpoint.get('Id', '')
                endpoint_arn = endpoint.get('Arn', '')
                endpoint_name = endpoint.get('Name', endpoint_id)

                details = {
                    'direction': endpoint.get('Direction'),
                    'status': endpoint.get('Status'),
                    'host_vpc_id': endpoint.get('HostVPCId'),
                    'ip_address_count': endpoint.get('IpAddressCount'),
                    'resolver_endpoint_type': endpoint.get('ResolverEndpointType'),
                }

                resources.append({
                    'service': 'route53resolver',
                    'type': 'resolver-endpoint',
                    'id': endpoint_id,
                    'arn': endpoint_arn,
                    'name': endpoint_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Resolver Rules
    try:
        paginator = resolver.get_paginator('list_resolver_rules')
        for page in paginator.paginate():
            for rule in page.get('ResolverRules', []):
                # Skip system rules
                if rule.get('OwnerId') == 'Route 53 Resolver':
                    continue

                rule_id = rule.get('Id', '')
                rule_arn = rule.get('Arn', '')
                rule_name = rule.get('Name', rule_id)

                details = {
                    'domain_name': rule.get('DomainName'),
                    'rule_type': rule.get('RuleType'),
                    'status': rule.get('Status'),
                    'resolver_endpoint_id': rule.get('ResolverEndpointId'),
                }

                resources.append({
                    'service': 'route53resolver',
                    'type': 'resolver-rule',
                    'id': rule_id,
                    'arn': rule_arn,
                    'name': rule_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Query Log Configs
    try:
        paginator = resolver.get_paginator('list_resolver_query_log_configs')
        for page in paginator.paginate():
            for config in page.get('ResolverQueryLogConfigs', []):
                config_id = config.get('Id', '')
                config_arn = config.get('Arn', '')
                config_name = config.get('Name', config_id)

                details = {
                    'status': config.get('Status'),
                    'destination_arn': config.get('DestinationArn'),
                    'association_count': config.get('AssociationCount'),
                }

                resources.append({
                    'service': 'route53resolver',
                    'type': 'query-log-config',
                    'id': config_id,
                    'arn': config_arn,
                    'name': config_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Firewall Rule Groups
    try:
        paginator = resolver.get_paginator('list_firewall_rule_groups')
        for page in paginator.paginate():
            for group in page.get('FirewallRuleGroups', []):
                # Skip shared rule groups we don't own
                if group.get('OwnerId') != account_id:
                    continue

                group_id = group.get('Id', '')
                group_arn = group.get('Arn', '')
                group_name = group.get('Name', group_id)

                details = {
                    'share_status': group.get('ShareStatus'),
                }

                resources.append({
                    'service': 'route53resolver',
                    'type': 'firewall-rule-group',
                    'id': group_id,
                    'arn': group_arn,
                    'name': group_name,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    # Firewall Domain Lists (skip AWS managed domain lists)
    try:
        paginator = resolver.get_paginator('list_firewall_domain_lists')
        for page in paginator.paginate():
            for domain_list in page.get('FirewallDomainLists', []):
                list_name = domain_list.get('Name', '')

                # Skip AWS managed domain lists (AWSManagedDomainsMalwareDomainList, etc.)
                if list_name.startswith('AWSManagedDomains'):
                    continue

                list_id = domain_list.get('Id', '')
                list_arn = domain_list.get('Arn', '')

                details = {
                    'managed_owner_name': domain_list.get('ManagedOwnerName'),
                }

                resources.append({
                    'service': 'route53resolver',
                    'type': 'firewall-domain-list',
                    'id': list_id,
                    'arn': list_arn,
                    'name': list_name or list_id,
                    'region': region,
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources



"""
AWS Global Accelerator resource collector.

Note: Global Accelerator is a global service that requires the us-west-2 region
for all API operations.
"""

import boto3
from typing import List, Dict, Any, Optional


def collect_globalaccelerator_resources(session: boto3.Session, region: Optional[str], account_id: str) -> List[Dict[str, Any]]:
    """
    Collect AWS Global Accelerator resources: accelerators, listeners, endpoint groups,
    custom routing accelerators, cross-account attachments, and BYOIP CIDRs.

    Note: Global Accelerator APIs must be called from us-west-2 region.

    Args:
        session: boto3.Session to use
        region: AWS region (ignored - always uses us-west-2)
        account_id: AWS account ID

    Returns:
        List of resource dictionaries
    """
    resources = []
    # Global Accelerator requires us-west-2 for all API calls
    ga = session.client('globalaccelerator', region_name='us-west-2')

    # Standard Accelerators
    try:
        paginator = ga.get_paginator('list_accelerators')
        for page in paginator.paginate():
            for accel in page.get('Accelerators', []):
                accel_arn = accel['AcceleratorArn']
                accel_name = accel.get('Name', accel_arn.split('/')[-1])

                details = {
                    'status': accel.get('Status'),
                    'enabled': accel.get('Enabled'),
                    'ip_address_type': accel.get('IpAddressType'),
                    'ip_sets': accel.get('IpSets', []),
                    'dns_name': accel.get('DnsName'),
                    'dual_stack_dns_name': accel.get('DualStackDnsName'),
                    'created_time': str(accel.get('CreatedTime', '')),
                    'last_modified_time': str(accel.get('LastModifiedTime', '')),
                }

                # Get accelerator attributes
                try:
                    attr_response = ga.describe_accelerator_attributes(AcceleratorArn=accel_arn)
                    attrs = attr_response.get('AcceleratorAttributes', {})
                    details['flow_logs_enabled'] = attrs.get('FlowLogsEnabled')
                    details['flow_logs_s3_bucket'] = attrs.get('FlowLogsS3Bucket')
                    details['flow_logs_s3_prefix'] = attrs.get('FlowLogsS3Prefix')
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = ga.list_tags_for_resource(ResourceArn=accel_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'globalaccelerator',
                    'type': 'accelerator',
                    'id': accel_arn.split('/')[-1],
                    'arn': accel_arn,
                    'name': accel_name,
                    'region': 'global',
                    'details': details,
                    'tags': tags
                })

                # Listeners for this accelerator
                try:
                    listener_paginator = ga.get_paginator('list_listeners')
                    for listener_page in listener_paginator.paginate(AcceleratorArn=accel_arn):
                        for listener in listener_page.get('Listeners', []):
                            listener_arn = listener['ListenerArn']

                            listener_details = {
                                'accelerator_arn': accel_arn,
                                'port_ranges': listener.get('PortRanges', []),
                                'protocol': listener.get('Protocol'),
                                'client_affinity': listener.get('ClientAffinity'),
                            }

                            resources.append({
                                'service': 'globalaccelerator',
                                'type': 'listener',
                                'id': listener_arn.split('/')[-1],
                                'arn': listener_arn,
                                'name': f"{accel_name}-listener-{listener_arn.split('/')[-1][:8]}",
                                'region': 'global',
                                'details': listener_details,
                                'tags': {}
                            })

                            # Endpoint Groups for this listener
                            try:
                                eg_paginator = ga.get_paginator('list_endpoint_groups')
                                for eg_page in eg_paginator.paginate(ListenerArn=listener_arn):
                                    for eg in eg_page.get('EndpointGroups', []):
                                        eg_arn = eg['EndpointGroupArn']
                                        eg_region = eg.get('EndpointGroupRegion', 'unknown')

                                        eg_details = {
                                            'listener_arn': listener_arn,
                                            'endpoint_group_region': eg_region,
                                            'traffic_dial_percentage': eg.get('TrafficDialPercentage'),
                                            'health_check_port': eg.get('HealthCheckPort'),
                                            'health_check_protocol': eg.get('HealthCheckProtocol'),
                                            'health_check_path': eg.get('HealthCheckPath'),
                                            'health_check_interval_seconds': eg.get('HealthCheckIntervalSeconds'),
                                            'threshold_count': eg.get('ThresholdCount'),
                                            'endpoints_count': len(eg.get('EndpointDescriptions', [])),
                                        }

                                        resources.append({
                                            'service': 'globalaccelerator',
                                            'type': 'endpoint-group',
                                            'id': eg_arn.split('/')[-1],
                                            'arn': eg_arn,
                                            'name': f"{accel_name}-{eg_region}",
                                            'region': 'global',
                                            'details': eg_details,
                                            'tags': {}
                                        })
                            except Exception:
                                pass
                except Exception:
                    pass
    except Exception:
        pass

    # Custom Routing Accelerators
    try:
        paginator = ga.get_paginator('list_custom_routing_accelerators')
        for page in paginator.paginate():
            for accel in page.get('Accelerators', []):
                accel_arn = accel['AcceleratorArn']
                accel_name = accel.get('Name', accel_arn.split('/')[-1])

                details = {
                    'status': accel.get('Status'),
                    'enabled': accel.get('Enabled'),
                    'ip_address_type': accel.get('IpAddressType'),
                    'ip_sets': accel.get('IpSets', []),
                    'dns_name': accel.get('DnsName'),
                    'created_time': str(accel.get('CreatedTime', '')),
                    'last_modified_time': str(accel.get('LastModifiedTime', '')),
                }

                # Get accelerator attributes
                try:
                    attr_response = ga.describe_custom_routing_accelerator_attributes(AcceleratorArn=accel_arn)
                    attrs = attr_response.get('AcceleratorAttributes', {})
                    details['flow_logs_enabled'] = attrs.get('FlowLogsEnabled')
                    details['flow_logs_s3_bucket'] = attrs.get('FlowLogsS3Bucket')
                    details['flow_logs_s3_prefix'] = attrs.get('FlowLogsS3Prefix')
                except Exception:
                    pass

                # Get tags
                tags = {}
                try:
                    tag_response = ga.list_tags_for_resource(ResourceArn=accel_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'globalaccelerator',
                    'type': 'custom-routing-accelerator',
                    'id': accel_arn.split('/')[-1],
                    'arn': accel_arn,
                    'name': accel_name,
                    'region': 'global',
                    'details': details,
                    'tags': tags
                })

                # Custom Routing Listeners
                try:
                    listener_paginator = ga.get_paginator('list_custom_routing_listeners')
                    for listener_page in listener_paginator.paginate(AcceleratorArn=accel_arn):
                        for listener in listener_page.get('Listeners', []):
                            listener_arn = listener['ListenerArn']

                            listener_details = {
                                'accelerator_arn': accel_arn,
                                'port_ranges': listener.get('PortRanges', []),
                            }

                            resources.append({
                                'service': 'globalaccelerator',
                                'type': 'custom-routing-listener',
                                'id': listener_arn.split('/')[-1],
                                'arn': listener_arn,
                                'name': f"{accel_name}-cr-listener-{listener_arn.split('/')[-1][:8]}",
                                'region': 'global',
                                'details': listener_details,
                                'tags': {}
                            })

                            # Custom Routing Endpoint Groups
                            try:
                                eg_paginator = ga.get_paginator('list_custom_routing_endpoint_groups')
                                for eg_page in eg_paginator.paginate(ListenerArn=listener_arn):
                                    for eg in eg_page.get('EndpointGroups', []):
                                        eg_arn = eg['EndpointGroupArn']
                                        eg_region = eg.get('EndpointGroupRegion', 'unknown')

                                        eg_details = {
                                            'listener_arn': listener_arn,
                                            'endpoint_group_region': eg_region,
                                            'destination_descriptions': eg.get('DestinationDescriptions', []),
                                            'endpoints_count': len(eg.get('EndpointDescriptions', [])),
                                        }

                                        resources.append({
                                            'service': 'globalaccelerator',
                                            'type': 'custom-routing-endpoint-group',
                                            'id': eg_arn.split('/')[-1],
                                            'arn': eg_arn,
                                            'name': f"{accel_name}-cr-{eg_region}",
                                            'region': 'global',
                                            'details': eg_details,
                                            'tags': {}
                                        })
                            except Exception:
                                pass
                except Exception:
                    pass
    except Exception:
        pass

    # Cross-Account Attachments
    try:
        paginator = ga.get_paginator('list_cross_account_attachments')
        for page in paginator.paginate():
            for attachment in page.get('CrossAccountAttachments', []):
                attachment_arn = attachment['AttachmentArn']
                attachment_name = attachment.get('Name', attachment_arn.split('/')[-1])

                details = {
                    'principals': attachment.get('Principals', []),
                    'resources': attachment.get('Resources', []),
                    'last_modified_time': str(attachment.get('LastModifiedTime', '')),
                    'created_time': str(attachment.get('CreatedTime', '')),
                }

                # Get tags
                tags = {}
                try:
                    tag_response = ga.list_tags_for_resource(ResourceArn=attachment_arn)
                    for tag in tag_response.get('Tags', []):
                        tags[tag.get('Key', '')] = tag.get('Value', '')
                except Exception:
                    pass

                resources.append({
                    'service': 'globalaccelerator',
                    'type': 'cross-account-attachment',
                    'id': attachment_arn.split('/')[-1],
                    'arn': attachment_arn,
                    'name': attachment_name,
                    'region': 'global',
                    'details': details,
                    'tags': tags
                })
    except Exception:
        pass

    # BYOIP CIDRs
    try:
        paginator = ga.get_paginator('list_byoip_cidrs')
        for page in paginator.paginate():
            for cidr in page.get('ByoipCidrs', []):
                cidr_block = cidr['Cidr']

                details = {
                    'state': cidr.get('State'),
                    'events': cidr.get('Events', []),
                }

                resources.append({
                    'service': 'globalaccelerator',
                    'type': 'byoip-cidr',
                    'id': cidr_block,
                    'arn': f"arn:aws:globalaccelerator::{account_id}:byoipcidr/{cidr_block}",
                    'name': cidr_block,
                    'region': 'global',
                    'details': details,
                    'tags': {}
                })
    except Exception:
        pass

    return resources
